@@ -26,7 +26,7 @@
 /*
  * Configuration structure, server, command and associated prototypes.
  *
- * $Id: dirtree.h,v 1.12 2001/09/26 15:34:10 flood Exp $
+ * $Id: dirtree.h,v 1.15 2002/05/21 20:47:15 castaglia Exp $
  */
 
 #ifndef __DIRTREE_H
@@ -63,6 +63,7 @@ typedef struct server_struc {
   struct conn_struc *listen;	/* Our listening connection */
   xaset_t *conf;		/* Configuration details */
 
+  int config_type;
 } server_rec;
 
 #define CLASS_USER
@@ -115,8 +116,9 @@ struct config_struc {
 
 /* config_rec flags */
 #define CF_MERGEDOWN		(1 << 0) /* Merge option down */
-#define CF_DYNAMIC		(1 << 1) /* Dynamically added entry */
-#define CF_DEFER		(1 << 2) /* Defer hashing until authentication */
+#define CF_MERGEDOWN_MULTI	(1 << 1) /* Merge down, allowing multiple instances */
+#define CF_DYNAMIC		(1 << 2) /* Dynamically added entry */
+#define CF_DEFER		(1 << 3) /* Defer hashing until authentication */
 
 /* Operation codes for dir_* funcs */
 #define OP_HIDE			1	/* Op for hiding dirs/files */
@@ -162,8 +164,8 @@ extern int			TimeoutStalled;
 #define CHECK_CONF(x,p)		if(!check_conf((x),(p))) \
 				CONF_ERROR((x), \
 				pstrcat((x)->tmp_pool,"directive not allowed in ", \
-				get_section_name((x)), \
-				" section.",NULL))
+				get_context_name((x)), \
+				" context",NULL))
 
 #define CHECK_CMD_ARGS(x, n)	if((x)->argc != (n)) { \
 				  add_response_err(R_501, \
@@ -183,56 +185,57 @@ extern int			TimeoutStalled;
  * Directory walking code will be completely redesigned in 1.3,
  * this is only necessary for perfomance reasons in 1.1/1.2
  */
-void kludge_disable_umask();
-void kludge_enable_umask();
+void kludge_disable_umask(void);
+void kludge_enable_umask(void);
 
-void init_config();
-void fixup_servers();
-int parse_config_file(const char*);
-config_rec *add_config_set(xaset_t**,const char*);
-config_rec *add_config(const char*);
-config_rec *add_config_param(const char*,int,...);
-config_rec *add_config_param_str(const char*,int,...);
-config_rec *add_config_param_set(xaset_t**,const char*,int,...);
-config_rec *find_config_next(config_rec*,config_rec*,int,const char*,int);
-config_rec *find_config(xaset_t*,int,const char*,int);
-void find_config_set_top(config_rec*);
-int remove_config(xaset_t*,const char*,int);
-class_t *find_class(p_in_addr_t *addr, char *remote_name);
+void init_config(void);
+void fixup_servers(void);
+int parse_config_file(const char *);
+config_rec *add_config_set(xaset_t **, const char *);
+config_rec *add_config(const char *);
+config_rec *add_config_param(const char *, int, ...);
+config_rec *add_config_param_str(const char *, int, ...);
+config_rec *add_config_param_set(xaset_t **, const char *, int, ...);
+config_rec *find_config_next(config_rec *, config_rec *, int,
+  const char *, int);
+config_rec *find_config(xaset_t *, int, const char *, int);
+void find_config_set_top(config_rec *);
+int remove_config(xaset_t *, const char *, int);
+class_t *find_class(p_in_addr_t *, char *);
 
-array_header *parse_group_expression(pool*,int*,char**);
-array_header *parse_user_expression(pool*,int*,char**);
-int group_expression(char**);
-int user_expression(char**);
+array_header *parse_group_expression(pool *, int *, char **);
+array_header *parse_user_expression(pool *, int *, char **);
+int group_expression(char **);
+int user_expression(char **);
 
-long get_param_int(xaset_t*,const char*,int);
-long get_param_int_next(const char*,int);
-void *get_param_ptr(xaset_t*,const char*,int);
-void *get_param_ptr_next(const char*,int);
-void init_conf_stacks();
-void free_conf_stacks();
-server_rec *start_new_server(const char*);
-server_rec *end_new_server();
-config_rec *start_sub_config(const char*);
-config_rec *end_sub_config();
-char *get_word(char**);
+long get_param_int(xaset_t *, const char *, int);
+long get_param_int_next(const char *, int);
+void *get_param_ptr(xaset_t *, const char *, int);
+void *get_param_ptr_next(const char *, int);
+void init_conf_stacks(void);
+void free_conf_stacks(void);
+server_rec *start_new_server(const char *);
+server_rec *end_new_server(void);
+config_rec *start_sub_config(const char *);
+config_rec *end_sub_config(void);
+char *get_word(char **);
 
-config_rec *dir_match_path(pool*,char*);
-void build_dyn_config(pool*,char*,struct stat*,int);
-int dir_check_hidden(const char *path);
-int dir_check_op_mode(pool*,char*,int,uid_t,gid_t,mode_t);
-int dir_check_full(pool*,char*,char*,char*,int*);
-int dir_check(pool*,char*,char*,char*,int*);
-int dir_check_canon(pool*,char*,char*,char*,int*);
+config_rec *dir_match_path(pool *, char *);
+void build_dyn_config(pool *, char *, struct stat *, int);
+int dir_check_hidden(const char *);
+int dir_check_op_mode(pool *, char *, int, uid_t, gid_t, mode_t);
+int dir_check_full(pool *, char *, char *, char *, int *);
+int dir_check(pool *, char *, char *, char *, int *);
+int dir_check_canon(pool *, char *, char *, char *, int *);
 int is_dotdir(const char *);
-int login_check_limits(xaset_t*,int,int,int*);
-void resolve_anonymous_dirs(xaset_t*);
-void resolve_defered_dirs(server_rec*);
-void fixup_dirs(server_rec*,int);
-int check_conf(cmd_rec*,int);
-char *get_section_name(cmd_rec*);
-int get_boolean(cmd_rec*,int);
-char *get_full_cmd(cmd_rec*);
-int match_ip(p_in_addr_t*, char*, const char *);
+int login_check_limits(xaset_t *, int, int, int *);
+void resolve_anonymous_dirs(xaset_t *);
+void resolve_defered_dirs(server_rec *);
+void fixup_dirs(server_rec *, int);
+unsigned char check_conf(cmd_rec *, int);
+char *get_context_name(cmd_rec *);
+int get_boolean(cmd_rec *, int);
+char *get_full_cmd(cmd_rec *);
+int match_ip(p_in_addr_t *, char *, const char *);
 
 #endif /* __DIRTREE_H */
