@@ -25,7 +25,7 @@
 
 /*
  * Authentication module for ProFTPD
- * $Id: mod_auth.c,v 1.62 2001/07/03 14:51:32 flood Exp $
+ * $Id: mod_auth.c,v 1.72 2002/05/21 20:47:16 castaglia Exp $
  */
 
 #include "conf.h"
@@ -40,26 +40,26 @@ module auth_module;
 static int logged_in = 0;
 static int auth_tries = 0;
 
-static void _do_user_counts();
+static void _do_user_counts(void);
 
 /* Perform a chroot or equivalent action to lockdown the process into a
  * particular directory.
  */
 static int lockdown(char *newroot)
 {
-  PRIVS_ROOT;
+  PRIVS_ROOT
   
   log_debug(DEBUG1, "Preparing to chroot() the environment, path = '%s'",
 	    newroot);
   
-  if(chroot(newroot) == -1) {
-    PRIVS_RELINQUISH;
+  if (chroot(newroot) == -1) {
+    PRIVS_RELINQUISH
     log_pri(LOG_ERR, "%s chroot(\"%s\"): %s", session.user,
 	    newroot, strerror(errno));
     return -1;
   }
   
-  PRIVS_RELINQUISH;
+  PRIVS_RELINQUISH
 
   log_debug(DEBUG1, "Environment successfully chroot()ed.");
 
@@ -104,20 +104,53 @@ int _login_timeout(CALLBACK_FRAME)
   return 0;		/* Don't restart the timer */
 }
 
-int auth_init_child()
-{
+static int auth_child_init(void) {
+  uid_t server_uid, current_euid = geteuid();
+  gid_t server_gid, current_egid = getegid();
+  unsigned char switch_server_id = FALSE;
+
   /* Start the login timer */
-  if(TimeoutLogin)
-    add_timer(TimeoutLogin,TIMER_LOGIN,&auth_module,_login_timeout);
-   
+  if (TimeoutLogin) {
+    remove_timer(TIMER_LOGIN, &auth_module);
+    add_timer(TimeoutLogin, TIMER_LOGIN, &auth_module, _login_timeout);
+  }
+
+  /* Set the privs for the configured User/Group of this server */
+  {
+    uid_t *uid = (uid_t *) get_param_ptr(main_server->conf, "UserID", FALSE);
+    gid_t *gid = (gid_t *) get_param_ptr(main_server->conf, "GroupID", FALSE);
+
+    if (uid) {
+      server_uid = *uid;
+      switch_server_id = TRUE;
+
+    } else
+      server_uid = current_euid;
+
+    if (gid) {
+      server_gid = *gid;
+      switch_server_id = TRUE;
+
+    } else
+      server_gid = current_egid;
+  }
+
+  if (switch_server_id) {
+    PRIVS_ROOT
+
+    /* Note: will it be necessary to double check this switch, as is done
+     * in src/main.c?
+     */
+    PRIVS_SETUP(server_uid, server_gid);
+  }
+
   if ((char*)get_param_ptr(main_server->conf,"DisplayConnect",FALSE) != NULL)
     _do_user_counts();
    
   return 0;
 }
 
-int auth_init()
-{
+static int auth_init(void) {
   /* By default, enable auth checking */
   set_auth_check(check_auth);
   return 0;
@@ -211,7 +244,7 @@ static config_rec *_auth_anonymous_group(pool *p, char *user)
    * may work properly
    */
   if (!session.gids && !session.groups &&
-      (ret = get_groups(p, user, &session.gids, &session.groups)) < 1)
+      (ret = auth_getgroups(p, user, &session.gids, &session.groups)) < 1)
     log_debug(DEBUG2, "no supplemental groups found for user '%s'", user);
 
   c = find_config(main_server->conf,CONF_PARAM,"AnonymousGroup",FALSE);
@@ -227,9 +260,9 @@ static config_rec *_auth_resolve_user(pool *p,char **user,
                                       char **ournamep,
                                       char **anonnamep)
 {
-  config_rec *c,*topc;
+  config_rec *c = NULL, *topc = NULL;
   char *ourname,*anonname = NULL;
-  int is_alias = 0, force_anon = 0;
+  unsigned char is_alias = FALSE, force_anon = FALSE;
 
   /* Precendence rules:
    *   1. Search for UserAlias directive.
@@ -242,24 +275,37 @@ static config_rec *_auth_resolve_user(pool *p,char **user,
   if(ournamep && ourname)
     *ournamep = ourname; 
 
-  c = find_config(main_server->conf,CONF_PARAM,"UserAlias",TRUE);
-  if(c) do {
-    if(!strcmp(c->argv[0], "*") || !strcmp(c->argv[0],*user)) {
-      is_alias = 1;
-      break;
-    }  
-  } while((c = find_config_next(c,c->next,CONF_PARAM,"UserAlias",TRUE)) != NULL);
+  c = find_config(main_server->conf, CONF_PARAM, "UserAlias", TRUE);
+
+  if (c)
+    do {
+      if (!strcmp(c->argv[0], "*") ||
+          !strcmp(c->argv[0], *user)) {
+        is_alias = TRUE;
+        break;
+      }  
+    } while ((c = find_config_next(c, c->next, CONF_PARAM, "UserAlias",
+      TRUE)) != NULL);
 
-  /* if AuthAliasOnly is set, ignore this one and continue */
   topc = c;
 
-  while(c && c->parent &&
-             find_config(c->parent->set,CONF_PARAM,"AuthAliasOnly",FALSE)) {
-    is_alias = 0;
+  while (c && c->parent) {
+    int *auth_alias_only = get_param_ptr(c->parent->set, "AuthAliasOnly",
+      FALSE);
+   
+    /* If AuthAliasOnly is on, ignore this one and continue. */
+    if (!auth_alias_only || *auth_alias_only != TRUE)
+      break;
+
+    is_alias = FALSE;
+
     find_config_set_top(topc);
-    c = find_config_next(c,c->next,CONF_PARAM,"UserAlias",TRUE);
-    if(c && (!strcmp(c->argv[0],"*") || !strcmp(c->argv[0],*user)))
-      is_alias = 1;
+    c = find_config_next(c, c->next, CONF_PARAM, "UserAlias", TRUE);
+
+    if (c &&
+        (!strcmp(c->argv[0], "*") ||
+         !strcmp(c->argv[0], *user)))
+      is_alias = TRUE;
   }
 
   if(c) {
@@ -301,16 +347,19 @@ static config_rec *_auth_resolve_user(pool *p,char **user,
       force_anon = 1;
   }
 
-  if(!is_alias && !force_anon) {
-    if(find_config((c ? c->subset :
-                   main_server->conf),CONF_PARAM,"AuthAliasOnly",FALSE)) {
-      
-      if(c && c->config_type == CONF_ANON)
+  if (!is_alias && !force_anon) {
+    int *auth_alias_only = get_param_ptr(c ? c->subset :
+      main_server->conf, "AuthAliasOnly", FALSE);
+
+    if (auth_alias_only && *auth_alias_only == TRUE) {
+      if (c && c->config_type == CONF_ANON)
         c = NULL;
       else
         *user = NULL;
 
-      if(*user && find_config(main_server->conf,CONF_PARAM,"AuthAliasOnly",FALSE))
+      auth_alias_only = get_param_ptr(main_server->conf, "AuthAliasOnly",
+        FALSE);
+      if (*user && auth_alias_only && *auth_alias_only == TRUE)
         *user = NULL;
 
       if((!user || !c) && anonnamep)
@@ -328,9 +377,9 @@ static int _auth_check_ftpusers(xaset_t *s, const char *user)
   char *u,buf[256] = {'\0'};
 
   if(get_param_int(s,"UseFtpUsers",FALSE) != 0) {
-    PRIVS_ROOT;
+    PRIVS_ROOT
     fp = fopen(FTPUSERS_PATH,"r");
-    PRIVS_RELINQUISH;
+    PRIVS_RELINQUISH
 
     if(!fp)
       return res;
@@ -457,11 +506,11 @@ static char *_get_default_root(pool *p)
       ** directive) as can not traverse down, we can still have the default
       ** root as ~/public_html/
       */
-      PRIVS_USER;
+      PRIVS_USER
 
       realdir = dir_realpath(p,dir);
 
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
 
       if(realdir)
         dir = realdir;
@@ -521,8 +570,9 @@ static int _setup_environment(pool *p, char *user, char *pass)
 
   origuser = user;
   c = _auth_resolve_user(p,&user,&ourname,&anonname);
+  if (c) session.anon_config = c;
 
-  if(!user) {
+  if (!user) {
     log_auth(LOG_NOTICE,"USER %s: user is not a UserAlias from %s [%s] to %s:%i",
              origuser,session.c->remote_name,
              inet_ascii(p,session.c->remote_ipaddr),
@@ -531,9 +581,6 @@ static int _setup_environment(pool *p, char *user, char *pass)
     goto auth_failure;
   }
 
-  /* If c != NULL from this point on, we have an anonymous login */
-  aclp = login_check_limits(main_server->conf,FALSE,TRUE,&i);
-
   if((pw = auth_getpwnam(p,user)) == NULL) {
     log_auth(LOG_NOTICE,"USER %s: no such user found from %s [%s] to %s:%i",
                user,session.c->remote_name,
@@ -549,15 +596,17 @@ static int _setup_environment(pool *p, char *user, char *pass)
 
   pw = passwd_dup(p,pw);
 
-  if(pw->pw_uid == 0) {
+  if (pw->pw_uid == 0) {
+    unsigned char *root_allow = NULL;
+ 
     /* If RootLogin is set to true, we allow this... even though we
      * still log a warning. :)
      */
-
-    if(get_param_int((c ? c->subset : main_server->conf),
-		     "RootLogin", FALSE) != 1) {
+    if ((root_allow = get_param_ptr(c ? c->subset : main_server->conf,
+        "RootLogin", FALSE)) == NULL || *root_allow != TRUE) {
       log_auth(LOG_CRIT, "SECURITY VIOLATION: root login attempted.");
       return 0;
+
     } else {
       log_auth(LOG_WARNING, "ROOT FTP login successful.");
     }
@@ -572,10 +621,13 @@ static int _setup_environment(pool *p, char *user, char *pass)
   
   /* Get the supplemental groups */
   if (!session.gids && !session.groups &&
-      (res = get_groups(p, pw->pw_name, &session.gids, &session.groups)) < 1)
+      (res = auth_getgroups(p, pw->pw_name, &session.gids,
+       &session.groups)) < 1)
     log_debug(DEBUG2, "no supplemental groups found for user '%s'",
       pw->pw_name);
 
+  /* If c != NULL from this point on, we have an anonymous login */
+  aclp = login_check_limits(main_server->conf,FALSE,TRUE,&i);
 
   /* set force_anon (for AnonymousGroup) and build a custom
    * anonymous config for this session.
@@ -654,8 +706,8 @@ static int _setup_environment(pool *p, char *user, char *pass)
         authcode = 0;
       }
     }
-      
-    bzero(pass,strlen(pass));
+    
+    memset(pass, '\0', strlen(pass));  
 
     switch(authcode) {
     case AUTH_NOPWD:
@@ -697,7 +749,8 @@ static int _setup_environment(pool *p, char *user, char *pass)
   auth_setgrent(p);
 
   if(!_auth_check_shell((c ? c->subset : main_server->conf),pw->pw_shell)) {
-    log_auth(LOG_NOTICE, "USER %s (Login failed): Invalid shell.", user);
+    log_auth(LOG_NOTICE, "USER %s (Login failed): Invalid shell: %s", user,
+      pw->pw_shell);
     goto auth_failure;
   }
 
@@ -725,7 +778,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
 
     block_signals();
     
-    PRIVS_ROOT;
+    PRIVS_ROOT
     if ((res = set_groups(p, pw->pw_gid, session.gids)) < 0)
       log_pri(LOG_ERR, "error: unable to set groups: %s",
         strerror(errno));
@@ -736,7 +789,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
     setuid(0);
     setgid(0);
 #endif
-    PRIVS_SETUP(pw->pw_uid,pw->pw_gid);
+    PRIVS_SETUP(pw->pw_uid, pw->pw_gid)
     
     if(add_userdir > 0 && strcmp(u, user))
       session.anon_root = dir_realpath(p, pdircat(p, c->name,
@@ -754,7 +807,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
       session.anon_root = pstrdup(session.pool,session.anon_root);
     
     /* return all privileges back to that of the daemon, for now */
-    PRIVS_ROOT;
+    PRIVS_ROOT
     if ((res = set_groups(p, daemon_gid, daemon_gids)) < 0)
       log_pri(LOG_ERR, "error: unable to set groups: %s",
         strerror(errno));
@@ -765,7 +818,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
     setuid(0);
     setgid(0);
 #endif
-    PRIVS_SETUP(daemon_uid, daemon_gid);
+    PRIVS_SETUP(daemon_uid, daemon_gid)
 
     unblock_signals();
     
@@ -774,7 +827,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
     if(getegid() != daemon_gid ||
        geteuid() != daemon_uid) {
 
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
       
       log_pri(LOG_ERR,"changing from %s back to daemon uid/gid: %s",
             session.user, strerror(errno));
@@ -880,7 +933,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
   if(wtmp_log == -1)
     wtmp_log = get_param_int(main_server->conf, "WtmpLog", FALSE);
 
-  PRIVS_ROOT;
+  PRIVS_ROOT
 
   if(wtmp_log != 0) {
     log_wtmp(ttyname, session.user, session.c->remote_name,
@@ -909,15 +962,16 @@ static int _setup_environment(pool *p, char *user, char *pass)
     log_pri(LOG_ERR, "error: unable to set groups: %s",
       strerror(errno));
 
-  PRIVS_RELINQUISH;
+  PRIVS_RELINQUISH
 
   /* Now check to see if the user has an applicable DefaultRoot */
   if(!c && (defroot = _get_default_root(session.pool))) {
 
     ensure_open_passwd(p);
 
-    if(lockdown(defroot) == -1) {
-      add_response_err(R_530, "Unable to set default root directory.");
+    if (lockdown(defroot) == -1) {
+      log_pri(LOG_ERR, "error: unable to set default root directory");
+      send_response(R_530, "Login incorrect.");
       end_login(1);
     }
 
@@ -940,8 +994,9 @@ static int _setup_environment(pool *p, char *user, char *pass)
   if(c)
     ensure_open_passwd(p);
 
-  if(c && lockdown(session.anon_root) == -1) {
-    add_response_err(R_530, "Unable to set anonymous privileges.");
+  if (c && lockdown(session.anon_root) == -1) {
+    log_pri(LOG_ERR, "error: unable to set anonymous privileges");
+    send_response(R_530, "Login incorrect.");
     end_login(1);
   }
 
@@ -952,29 +1007,28 @@ static int _setup_environment(pool *p, char *user, char *pass)
 #ifndef __hpux
   block_signals();
 
-  PRIVS_ROOT;
+  PRIVS_ROOT
 
   setuid(0);
   setgid(0);
 
-  PRIVS_SETUP(pw->pw_uid,pw->pw_gid);
+  PRIVS_SETUP(pw->pw_uid, pw->pw_gid)
 
   unblock_signals();
 #else
   session.uid = session.ouid = pw->pw_uid;
   session.gid = pw->pw_gid;
-  PRIVS_RELINQUISH;
+  PRIVS_RELINQUISH
 #endif
 
 #ifdef HAVE_GETEUID
   if(getegid() != pw->pw_gid ||
      geteuid() != pw->pw_uid) {
 
-    PRIVS_RELINQUISH;
-
-    add_response_err(R_530, "Unable to set user privileges.");
-    log_pri(LOG_ERR, "%s setregid() or setreuid(): %s",
+    PRIVS_RELINQUISH
+    log_pri(LOG_ERR, "error: %s setregid() or setreuid(): %s",
             session.user, strerror(errno));
+    send_response(R_530, "Login incorrect.");
 
     end_login(1);
   }
@@ -1008,6 +1062,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
   if (pw->pw_dir == NULL || !strcmp(pw->pw_dir, "")) {
     log_pri(LOG_ERR, "error: user %s home directory is NULL or \"\"",
       session.user);
+    send_response(R_530, "Login incorrect.");
     end_login(1);
   }
 
@@ -1034,23 +1089,28 @@ static int _setup_environment(pool *p, char *user, char *pass)
       }
 
     } else if(defchdir) {
-    /* if we've got defchdir, failure is ok as well, simply switch to
-     * user's homedir.
-     */
-      log_debug(DEBUG2, "unable to chdir to %s, defaulting to home directory %s",
-        session.cwd, pw->pw_dir);
+
+      /* If we've got defchdir, failure is ok as well, simply switch to
+       * user's homedir.
+       */
+      log_debug(DEBUG2, "unable to chdir to %s, defaulting to home "
+        "directory %s", session.cwd, pw->pw_dir);
 
       if (fs_chdir_canon(pw->pw_dir, !showsymlinks) == -1) {
-          log_pri(LOG_ERR, "%s chdir(\"%s\"): %s", session.user, session.cwd,
+        log_pri(LOG_ERR, "%s chdir(\"%s\"): %s", session.user, session.cwd,
                   strerror(errno));
-          send_response(R_530,"Login incorrect.");
-          end_login(1);
+        send_response(R_530, "Login incorrect.");
+        end_login(1);
       }
+
     } else {
-    /* can't switch to user's real home directory, which is not allowed. */
+
+      /* Unable to switch to user's real home directory, which is not
+       * allowed.
+       */
       log_pri(LOG_ERR, "%s chdir(\"%s\"): %s", session.user, session.cwd,
               strerror(errno));
-      send_response(R_530,"Login incorrect.");
+      send_response(R_530, "Login incorrect.");
       end_login(1);
     }
   }
@@ -1062,6 +1122,9 @@ static int _setup_environment(pool *p, char *user, char *pass)
   if (fs_stat(session.cwd, &sbuf) != -1)
     build_dyn_config(p, session.cwd, &sbuf, 1);
 
+  /* make sure session.dir_config is set correctly */
+  dir_check_full(p,C_PASS,G_NONE,session.cwd,NULL);
+
   if(c) {
     if(!session.hide_password)
       session.proc_prefix =
@@ -1072,11 +1135,11 @@ static int _setup_environment(pool *p, char *user, char *pass)
       pstrcat(permanent_pool,session.c->remote_name,
               ": anonymous",NULL);
 
-    session.anon_config = c;
     session.flags = SF_ANON;
+
   } else {
+
     session.proc_prefix = pstrdup(permanent_pool,session.c->remote_name);
-            
     session.flags = 0;
   }
 
@@ -1102,7 +1165,7 @@ static int _setup_environment(pool *p, char *user, char *pass)
   log_run_cwd(session.cwd);
   main_set_idle();
 
-  remove_timer(TIMER_LOGIN,&auth_module);
+  remove_timer(TIMER_LOGIN, &auth_module);
 
   /* these copies are made from the permanent_pool, instead of the more
    * volatile pool used originally, in order that the copied data maintain
@@ -1130,7 +1193,7 @@ auth_failure:
    CURRENT_CLASS based counters and an estimate for the number of clients. The
    primary prupose is to make it so that the %N/%y escapes work in a 
    DisplayConnect greeting */
-static void _do_user_counts()
+static void _do_user_counts(void)
 {
   logrun_t *l;
   int cur = -1, ccur = -1;
@@ -1144,7 +1207,7 @@ static void _do_user_counts()
     return;
   
   /* Determine how many users are currently connected */
-  PRIVS_ROOT;
+  PRIVS_ROOT
   while((l = log_read_run(NULL)) != NULL)
       /* Make sure it matches our current server */
       if(l->server_ip.s_addr == main_server->ipaddr->s_addr &&
@@ -1154,7 +1217,7 @@ static void _do_user_counts()
         if(strcasecmp(l->class, session.class->name) == 0)
         	ccur++;
       }
-  PRIVS_RELINQUISH;
+  PRIVS_RELINQUISH
   
   /* This silliness is needed to get past the broken HP/UX 11.x compiler.
    */
@@ -1200,7 +1263,7 @@ static void _auth_check_count(cmd_rec *cmd, char *user) {
   /* Gather our statistics.
    */
   if(user) {
-    PRIVS_ROOT;
+    PRIVS_ROOT
     
     while((l = log_read_run(NULL)) != NULL) {
       int samehost = 0;
@@ -1251,7 +1314,7 @@ static void _auth_check_count(cmd_rec *cmd, char *user) {
       }
     }
     
-    PRIVS_RELINQUISH;
+    PRIVS_RELINQUISH
   }
   
   remove_config(cmd->server->conf, "CURRENT-CLIENTS", FALSE);
@@ -1303,7 +1366,7 @@ static void _auth_check_count(cmd_rec *cmd, char *user) {
   maxc = find_config((c ? c->subset : cmd->server->conf),
 		     CONF_PARAM, "MaxClientsPerHost", FALSE);
   
-  if(maxc) {
+  if(maxc && (int)maxc->argv[0] != -1) {
     int max = (int) maxc->argv[0];
     char *maxstr = "Sorry, the maximum number clients (%m) from your host are "
       "already connected.";
@@ -1331,7 +1394,7 @@ static void _auth_check_count(cmd_rec *cmd, char *user) {
   maxc = find_config((c ? c->subset : cmd->server->conf),
 		     CONF_PARAM, "MaxClients", FALSE);
 
-  if(maxc) {
+  if(maxc && ((int)maxc->argv[0] != -1)) {
     int max = (int) maxc->argv[0];
     char *maxstr = "Sorry, the maximum number of allowed clients (%m) "
       "already connected.";
@@ -1358,7 +1421,7 @@ static void _auth_check_count(cmd_rec *cmd, char *user) {
   maxc = find_config((c ? c->subset : cmd->server->conf),
 		     CONF_PARAM, "MaxHostsPerUser", FALSE);
   
-  if(maxc) {
+  if(maxc && (int)maxc->argv[0] != -1) {
     int max = (int) maxc->argv[0];
     char *maxstr = "Sorry, the maximum number of hosts (%m) for this user "
       "already connected.";
@@ -1609,16 +1672,42 @@ MODRET cmd_rein(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
+/* Configuration handlers
+ */
+
+MODRET set_authaliasonly(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected Boolean parameter");
+
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(int));
+  *((int *) c->argv[0]) = bool;
+
+  return HANDLED(cmd);
+}
 
 MODRET set_rootlogin(cmd_rec *cmd) {
-  int bool;
+  int bool = -1;
+  config_rec *c = NULL;
+
   CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
   
   if ((bool = get_boolean(cmd, 1)) == -1)
     CONF_ERROR(cmd, "expected boolean argument.");
  
-  add_config_param("RootLogin", 1, (void*) bool);
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
+  *((unsigned char *) c->argv[0]) = (unsigned char) bool;
+
+  c->flags |= CF_MERGEDOWN;
+
   return HANDLED(cmd);
 }
 
@@ -1740,16 +1829,17 @@ MODRET add_userdirroot (cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-static conftable auth_config[] = {
-  { "RootLogin",		set_rootlogin,			NULL },
-  { "LoginPasswordPrompt",	set_loginpasswordprompt,	NULL },
+static conftable auth_conftab[] = {
+  { "AuthAliasOnly",		set_authaliasonly,		NULL },
   { "DefaultRoot",		add_defaultroot,		NULL },
   { "DefaultChdir",		add_defaultchdir,		NULL },
+  { "LoginPasswordPrompt",	set_loginpasswordprompt,	NULL },
+  { "RootLogin",		set_rootlogin,			NULL },
   { "UserDirRoot",		add_userdirroot,		NULL },
   { NULL,			NULL,				NULL }
 };
 
-static cmdtable auth_commands[] = {
+static cmdtable auth_cmdtab[] = {
   { PRE_CMD, C_USER, G_NONE, pre_cmd_user, FALSE, FALSE, CL_AUTH },
   { CMD, C_USER, G_NONE, cmd_user,	FALSE,	FALSE, CL_AUTH },
   { PRE_CMD, C_PASS, G_NONE, pre_cmd_pass, FALSE, FALSE, CL_AUTH },
@@ -1765,9 +1855,10 @@ module auth_module = {
   NULL,NULL,				/* Always NULL */
   0x20,					/* API Version 2.0 */
   "auth",
-  auth_config,	
-  auth_commands,
+  auth_conftab,	
+  auth_cmdtab,
   NULL,
-  auth_init,auth_init_child
+  auth_init,
+  auth_child_init
 };
 
