@@ -25,7 +25,7 @@
 
 /*
  * Core FTPD module
- * $Id: mod_core.c,v 1.73 2001/10/19 15:27:58 flood Exp $
+ * $Id: mod_core.c,v 1.84 2002/05/21 20:47:16 castaglia Exp $
  *
  * 11/5/98	Habeeb J. Dihu aka MacGyver (macgyver@tos.net): added
  * 			wu-ftpd style CDPath support.
@@ -110,7 +110,7 @@ static ssize_t get_num_bytes(char *nbytes_str) {
   char units, junk;
   int result;
 
-  /* scan in the given argument, checking for the leading number-of-bytes
+  /* Scan in the given argument, checking for the leading number-of-bytes
    * as well as a trailing G, M, K, or B (case-insensitive).  The junk
    * variable is catch arguments like "2g2" or "number-letter-whatever".
    *
@@ -129,7 +129,8 @@ static ssize_t get_num_bytes(char *nbytes_str) {
       return PR_BYTES_BAD_UNITS;
 
     nbytes = (ssize_t)inb;
-    /* calculate the actual bytes, multiplying by the given units.  Doing
+
+    /* Calculate the actual bytes, multiplying by the given units.  Doing
      * it this way means that <math.h> and -lm aren't required.
      */
     if (units == 'G' || units == 'g')
@@ -141,7 +142,7 @@ static ssize_t get_num_bytes(char *nbytes_str) {
     if (units == 'K' || units == 'k')
       nbytes *= 1024;
 
-    /* silently ignore units of 'B' and 'b', as they don't affect
+    /* Silently ignore units of 'B' and 'b', as they don't affect
      * the requested number of bytes anyway.
      */
 
@@ -153,29 +154,21 @@ static ssize_t get_num_bytes(char *nbytes_str) {
 
   } else if (result == 1) {
 
-    /* no units given -- just return the number of bytes as is
-     */
+    /* No units given -- just return the number of bytes as is. */
     return nbytes;
-
-  } else {
-
-    /* the given argument was badly formatted.
-     */
-    return PR_BYTES_BAD_FORMAT;
   }
 
-  /* default value */
-  return 0;
+  /* Default return value: the given argument was badly formatted.
+   */
+  return PR_BYTES_BAD_FORMAT;
 }
 
 MODRET add_include(cmd_rec *cmd)
 {
   CHECK_ARGS(cmd, 1);
-  CHECK_CONF(cmd, CONF_ROOT | CONF_DIR | CONF_ANON | CONF_LIMIT |
-    CONF_VIRTUAL | CONF_GLOBAL);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL|CONF_DIR);
   
-  /* make sure the given path is a full path, not a relative one -- TJ
-   */
+  /* make sure the given path is a full path, not a relative one */
   if (*(cmd->argv[1]) != '/') {
     CONF_ERROR(cmd, pstrcat(cmd->tmp_pool,
       "Unable to use relative path for configuration file '",
@@ -326,7 +319,7 @@ MODRET set_deferwelcome(cmd_rec *cmd)
 
 MODRET set_pidfile(cmd_rec *cmd) {
   CHECK_ARGS(cmd, 1);
-  CHECK_CONF(cmd, CONF_ROOT | CONF_GLOBAL);
+  CHECK_CONF(cmd, CONF_ROOT);
 
   add_config_param_str("PidFile", 1, cmd->argv[1]);
   return HANDLED(cmd);
@@ -407,16 +400,14 @@ MODRET set_defaultserver(cmd_rec *cmd)
 }
 
 MODRET add_masqueradeaddress(cmd_rec *cmd) {
-
  config_rec *c = NULL;
- p_in_addr_t *masq_addr;
+ p_in_addr_t *masq_addr = NULL;
  char masq_ip[80] = {'\0'};
 
  CHECK_ARGS(cmd, 1);
- CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
+ CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL);
 
- /* make a copy of the given argument
-  */
+ /* Make a copy of the given argument.  */
  sstrncpy(masq_ip, cmd->argv[1], sizeof(masq_ip));
 
  if ((masq_addr = inet_getaddr(cmd->server->pool, masq_ip)) == NULL)
@@ -424,7 +415,7 @@ MODRET add_masqueradeaddress(cmd_rec *cmd) {
      (cmd->argv)[0], ": unable to resolve \"", masq_ip, "\"",
      NULL));
 
- c = add_config_param("MasqueradeAddress", 1, (void *) masq_addr);
+ c = add_config_param(cmd->argv[0], 1, (void *) masq_addr);
  return HANDLED(cmd);
 }
 
@@ -645,16 +636,20 @@ MODRET set_multilinerfc2228(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
-MODRET set_identlookups(cmd_rec *cmd)
-{
-  int bool;
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
+MODRET set_identlookups(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
 
   if ((bool = get_boolean(cmd, 1)) == -1)
     CONF_ERROR(cmd, "expected boolean argument.");
 
-  add_config_param("IdentLookups", 1, bool);
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
+  *((unsigned char *) c->argv[0]) = bool;
+
   return HANDLED(cmd);
 }
 
@@ -721,56 +716,59 @@ MODRET set_tcpnodelay(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
-MODRET set_user(cmd_rec *cmd)
-{
+MODRET set_user(cmd_rec *cmd) {
   struct passwd *pw = NULL;
 
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
 
   /* 1.1.7, no longer force user/group lookup inside <Anonymous>
    * it's now defered until authentication occurs.
    */
 
-  if(!cmd->config || cmd->config->config_type != CONF_ANON) {
-    if((pw = auth_getpwnam(cmd->tmp_pool,cmd->argv[1])) == NULL) {
+  if (!cmd->config || cmd->config->config_type != CONF_ANON) {
+    if ((pw = auth_getpwnam(cmd->tmp_pool, cmd->argv[1])) == NULL) {
       auth_endpwent(cmd->tmp_pool);
-      CONF_ERROR(cmd,pstrcat(cmd->tmp_pool,"Unknown user '",
-                             cmd->argv[1],"'.",NULL));
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "Unknown user '",
+        cmd->argv[1], "'.", NULL));
     }
   }
 
-  if(pw) {
-    add_config_param("User",1,(void*)pw->pw_uid);
+  if (pw) {
+    config_rec *c = add_config_param("UserID", 1, NULL);
+    c->argv[0] = pcalloc(c->pool, sizeof(uid_t));
+    *((uid_t *) c->argv[0]) = pw->pw_uid;
+
     /* We don't need extra fds sitting around open */
     auth_endpwent(cmd->tmp_pool);
   }
 
-  add_config_param_str("UserName",1,(void*)cmd->argv[1]);
+  add_config_param_str("UserName", 1, cmd->argv[1]);
   return HANDLED(cmd);
 }
 
-MODRET set_group(cmd_rec *cmd)
-{
+MODRET set_group(cmd_rec *cmd) {
   struct group *grp = NULL;
 
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
 
-  if(!cmd->config || cmd->config->config_type != CONF_ANON) {
-    if((grp = auth_getgrnam(cmd->tmp_pool,cmd->argv[1])) == NULL) {
+  if (!cmd->config || cmd->config->config_type != CONF_ANON) {
+    if ((grp = auth_getgrnam(cmd->tmp_pool, cmd->argv[1])) == NULL) {
       auth_endgrent(cmd->tmp_pool);
-      CONF_ERROR(cmd,pstrcat(cmd->tmp_pool,"Unknown group '",
-                             cmd->argv[1],"'.",NULL));
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "Unknown group '",
+        cmd->argv[1], "'.", NULL));
     }
   }
 
-  if(grp) {
-    add_config_param("Group",1,(void*)grp->gr_gid);
+  if (grp) {
+    config_rec *c = add_config_param("GroupID", 1, NULL);
+    c->argv[0] = pcalloc(c->pool, sizeof(gid_t));
+    *((gid_t *) c->argv[0]) = grp->gr_gid;
     auth_endgrent(cmd->tmp_pool);
   }
-  add_config_param_str("GroupName",1,(void*)cmd->argv[1]);
 
+  add_config_param_str("GroupName", 1, cmd->argv[1]);
   return HANDLED(cmd);
 }
 
@@ -800,8 +798,7 @@ MODRET set_accessdenymsg(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-MODRET set_accessgrantmsg(cmd_rec *cmd)
-{
+MODRET set_accessgrantmsg(cmd_rec *cmd) {
   CHECK_ARGS(cmd,1);
   CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
 
@@ -813,19 +810,21 @@ MODRET set_umask(cmd_rec *cmd)
 {
   config_rec *c;
   char *endp;
-  mode_t *_umask = (mode_t *) pcalloc(cmd->server->pool, sizeof(mode_t));
-  
+  mode_t tmp_umask;
+ 
   CHECK_VARARGS(cmd, 1, 2);
   CHECK_CONF(cmd, CONF_ROOT | CONF_VIRTUAL | CONF_DIR |
 	     CONF_ANON | CONF_GLOBAL | CONF_DYNDIR);
   
-  *_umask = (mode_t) strtol(cmd->argv[1], &endp, 8);
+  tmp_umask = (mode_t) strtol(cmd->argv[1], &endp, 8);
   
   if(endp && *endp)
     CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "'", cmd->argv[1], "' is not "
 			    "a valid umask.", NULL));
   
-  c = add_config_param("Umask", 1, (void *) _umask);
+  c = add_config_param("Umask", 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(mode_t));
+  *((mode_t *) c->argv[0]) = tmp_umask;
   c->flags |= CF_MERGEDOWN;
   
   /* Have we specified a directory umask as well?
@@ -835,14 +834,15 @@ MODRET set_umask(cmd_rec *cmd)
     /* allocate space for another mode_t.  Don't worry -- the previous
      * pointer was recorded in the Umask config_rec
      */
-    _umask = (mode_t *) pcalloc(cmd->server->pool, sizeof(mode_t));
-    *_umask = (mode_t) strtol(cmd->argv[2], &endp, 8);
+    tmp_umask = (mode_t) strtol(cmd->argv[2], &endp, 8);
     
     if(endp && *endp)
       CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "'", cmd->argv[2], "' is not "
 			      "a valid umask.", NULL));
     
-    c = add_config_param("DirUmask", 1, (void *) _umask);
+    c = add_config_param("DirUmask", 1, NULL);
+    c->argv[0] = pcalloc(c->pool, sizeof(mode_t));
+    *((mode_t *) c->argv[0]) = tmp_umask;
     c->flags |= CF_MERGEDOWN;
   }
   
@@ -1057,16 +1057,24 @@ MODRET set_syslogfacility(cmd_rec *cmd)
       block_signals();
       PRIVS_ROOT
 	switch(log_opensyslog(NULL)) {
-	case -2:
+
+        case -1:
+          PRIVS_RELINQUISH
+          unblock_signals();
+          CONF_ERROR(cmd, "unable to open syslog");
+          break;
+
+	case LOG_WRITEABLE_DIR:
 	  PRIVS_RELINQUISH
 	  unblock_signals();
-	  CONF_ERROR(cmd, "you are attempting to log to a world writeable directory");
+	  CONF_ERROR(cmd,
+            "you are attempting to log to a world writeable directory");
 	  break;
 	  
-	case -1:
+	case LOG_SYMLINK:
 	  PRIVS_RELINQUISH
 	  unblock_signals();
-	  CONF_ERROR(cmd, "unable to open syslog");
+	  CONF_ERROR(cmd, "you are attempting to log to a symbolic link");
 	  break;
 	  
 	default:
@@ -1399,38 +1407,45 @@ MODRET add_anonymousgroup(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
-MODRET add_hideuser(cmd_rec *cmd)
-{
-  config_rec *c;
-  struct passwd *pw;
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_DIR|CONF_ANON);
+MODRET add_hideuser(cmd_rec *cmd) {
+  config_rec *c = NULL;
+  struct passwd *pw = NULL;
 
-  pw = auth_getpwnam(cmd->tmp_pool,cmd->argv[1]);
-  if(!pw)
-    CONF_ERROR(cmd,pstrcat(cmd->tmp_pool,"'",cmd->argv[1],"' is not "
-                   "a valid user.",NULL));
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_DIR|CONF_ANON);
 
-  c = add_config_param("HideUser",1,(void*)((int)pw->pw_uid));
+  pw = auth_getpwnam(cmd->tmp_pool, cmd->argv[1]);
+
+  if (!pw)
+    CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "'", cmd->argv[1],
+      "' is not a valid user.", NULL));
+
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(uid_t));
+  *((uid_t *) c->argv[0]) = pw->pw_uid;
   c->flags |= CF_MERGEDOWN;
+
   return HANDLED(cmd);
 }
 
-MODRET add_hidegroup(cmd_rec *cmd)
-{
-  config_rec *c;
-  struct group *gr;
+MODRET add_hidegroup(cmd_rec *cmd) {
+  config_rec *c = NULL;
+  struct group *gr = NULL;
 
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_DIR|CONF_ANON);
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_DIR|CONF_ANON);
 
-  gr = auth_getgrnam(cmd->tmp_pool,cmd->argv[1]);
-  if(!gr)
-    CONF_ERROR(cmd,pstrcat(cmd->tmp_pool,"'",cmd->argv[1],"' is not "
-                   "a valid group.",NULL));
+  gr = auth_getgrnam(cmd->tmp_pool, cmd->argv[1]);
 
-  c = add_config_param("HideGroup",1,(void*)((int)gr->gr_gid));
+  if (!gr)
+    CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "'", cmd->argv[1],
+      "' is not a valid group.", NULL));
+
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(gid_t));
+  *((gid_t *) c->argv[0]) = gr->gr_gid;
   c->flags |= CF_MERGEDOWN;
+
   return HANDLED(cmd);
 }
 
@@ -1577,7 +1592,7 @@ MODRET add_global(cmd_rec *cmd)
   CHECK_ARGS(cmd,0);
   CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL);
 
-  c = start_sub_config("Global");
+  c = start_sub_config("<Global>");
   c->config_type = CONF_GLOBAL;
 
   return HANDLED(cmd);
@@ -1912,21 +1927,6 @@ MODRET set_displaygoaway(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
-MODRET set_authaliasonly(cmd_rec *cmd)
-{
-  int b;
-
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
-
-  if((b = get_boolean(cmd,1)) == -1)
-    CONF_ERROR(cmd,"expected boolean argument.");
-
-  add_config_param("AuthAliasOnly",1,(void*)b);
-
-  return HANDLED(cmd);
-}
-
 MODRET add_virtualhost(cmd_rec *cmd)
 {
   server_rec *s;
@@ -1934,9 +1934,6 @@ MODRET add_virtualhost(cmd_rec *cmd)
   CHECK_ARGS(cmd,1);
   CHECK_CONF(cmd,CONF_ROOT);
 
-  if(cmd->server != main_server)
-    CONF_ERROR(cmd,"directive cannot be nested.");
-
   s = start_new_server(cmd->argv[1]);
   if(!s)
     CONF_ERROR(cmd,"unable to create virtual server configuration.");
@@ -1947,9 +1944,7 @@ MODRET add_virtualhost(cmd_rec *cmd)
 MODRET end_virtualhost(cmd_rec *cmd)
 {
   CHECK_ARGS(cmd,0);
-
-  if(cmd->server == main_server)
-    CONF_ERROR(cmd,"must be matched with <VirtualHost> directive.");
+  CHECK_CONF(cmd, CONF_VIRTUAL);
 
   end_new_server();
   return HANDLED(cmd);
@@ -2091,23 +2086,31 @@ MODRET regex_filters(cmd_rec *cmd)
    * we are receiving commands, so it's ok to cache the regex for performance.
    */
   static regex_t *a_reg = NULL;
-  static int a_reg_cached = 0;
+  static int a_reg_cached = FALSE;
   static regex_t *d_reg = NULL;
-  static int d_reg_cached = 0;
+  static int d_reg_cached = FALSE;
   int ret;
-  
+ 
+  /* if authenticated, do lookups again.  This allows {Allow,Deny}Filter to
+   * operate on the USER command (although I don't know why you'd want that)
+   */
+  if (find_config(cmd->server->conf, CONF_PARAM, "authenticated", FALSE)) {
+    a_reg_cached = FALSE;
+    d_reg_cached = FALSE;
+  }
+
+  /* Don't apply the filter checks to passwords (arguments to the PASS
+   * command).
+   */
+  if (strcasecmp(cmd->argv[0], "PASS") == 0)
+    return DECLINED(cmd);
+
   /* Check for an AllowFilter */
   if(!a_reg_cached) {
     a_reg = (regex_t*) get_param_ptr(TOPLEVEL_CONF, "AllowFilter", FALSE);
     a_reg_cached = TRUE;
   }
   
-  /* Don't apply the filter checks to passwords (arguments to the PASS
-   * command).
-   */
-  if(strcasecmp(cmd->argv[0],"PASS") == 0)
-    return DECLINED(cmd);
-
   if(a_reg && cmd->arg &&
      ((ret = regexec(a_reg, cmd->arg, 0, NULL, 0)) != 0)) {
     log_debug(DEBUG2, "'%s %s' denied by AllowFilter", cmd->argv[0],
@@ -2362,7 +2365,7 @@ MODRET cmd_help(cmd_rec *cmd)
     char *outa[8];
     char *outs = "";
 
-    bzero(outa,sizeof(outa));
+    memset(outa, '\0', sizeof(outa));
 
     add_response(R_214,
       "The following commands are recognized (* =>'s unimplemented).");
@@ -2390,7 +2393,7 @@ MODRET cmd_help(cmd_rec *cmd)
           add_response(R_214,"%s",outs);
         outs = "";
         c = 0;
-        bzero(outa,sizeof(outa));
+        memset(outa, '\0', sizeof(outa));
       }
     }
 
@@ -2542,7 +2545,7 @@ MODRET _chdir(cmd_rec *cmd,char *ndir)
       core_display_file(R_250,display,session.cwd);
   }
 
-  add_response(R_250,"CWD command successful.");
+  add_response(R_250,"%s command successful.", cmd->argv[0]);
   return HANDLED(cmd);
 }
 
@@ -2626,51 +2629,51 @@ MODRET cmd_mkd(cmd_rec *cmd)
 
   dir = dir_canonical_path(cmd->tmp_pool,cmd->arg);
 
-  if(!dir || !dir_check_canon(cmd->tmp_pool,cmd->argv[0],cmd->group,dir,NULL) ||
-     mkdir(dir,0777) == -1) {
-    add_response_err(R_550,"%s: %s",cmd->argv[1],strerror(errno));
+  if (!dir ||
+      !dir_check_canon(cmd->tmp_pool, cmd->argv[0], cmd->group,dir, NULL) ||
+       mkdir(dir, 0777) == -1) {
+    add_response_err(R_550, "%s: %s", cmd->argv[1], strerror(errno));
     return ERROR(cmd);
-  } else {
 
-    if(session.fsuid != -1) {
+  } else {
+    if (session.fsuid != (uid_t) -1) {
       int err = 0,iserr = 0;
 
-      fs_stat(dir,&sbuf);
+      fs_stat(dir, &sbuf);
 
-      PRIVS_ROOT;
-      if(fs_chown(dir,session.fsuid,session.fsgid) == -1) {
+      PRIVS_ROOT
+      if (fs_chown(dir, session.fsuid, session.fsgid) == -1) {
         iserr++;
         err = errno;
       }
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
+
+      if (iserr)
+        log_pri(LOG_WARNING, "chown() as root failed: %s", strerror(err));
 
-      if(iserr)
-        log_pri(LOG_WARNING, "chown() as root failed: %s.", strerror(err));
       else {
-        if(session.fsgid != -1)
+        if (session.fsgid != (gid_t) -1)
           log_debug(DEBUG2, "root chown(%s) to uid %lu, gid %lu successful",
-                    dir,
-                    (unsigned long)session.fsuid,
-                    (unsigned long)session.fsgid);
+            dir, (unsigned long) session.fsuid, (unsigned long) session.fsgid);
+
         else
-          log_debug(DEBUG2, "root chown(%s) to uid %lu successful",
-                    dir,
-                    (unsigned long)session.fsuid);
-        fs_chmod(dir,sbuf.st_mode);
-      }
-    } else if(session.fsgid != -1) {
-      fs_stat(dir,&sbuf);
-      if(fs_chown(dir,(uid_t)-1,(gid_t)session.fsgid) == -1)
-        log_pri(LOG_WARNING, "chown() failed: %s.", strerror(errno));
-      else {
-        log_debug(DEBUG2, "chown(%s) to gid %lu successful",
-                  dir,
-                  (unsigned long)session.fsgid);
-        fs_chmod(dir,sbuf.st_mode);
+          log_debug(DEBUG2, "root chown(%s) to uid %lu successful", dir,
+            (unsigned long) session.fsuid);
       }
+
+    } else if (session.fsgid != (gid_t) -1) {
+      fs_stat(dir, &sbuf);
+
+      if (fs_chown(dir, (uid_t) -1, session.fsgid) == -1)
+        log_pri(LOG_WARNING, "chown() failed: %s", strerror(errno));
+
+      else
+        log_debug(DEBUG2, "chown(%s) to gid %lu successful", dir,
+          (unsigned long) session.fsgid);
     }
-    add_response(R_257,"\"%s\" - Directory successfully created.",
-                  quote_dir(cmd,dir));
+
+    add_response(R_257, "\"%s\" - Directory successfully created",
+      quote_dir(cmd, dir));
   }
     
   return HANDLED(cmd);
@@ -2849,7 +2852,7 @@ MODRET cmd_rnto(cmd_rec *cmd)
   if(!session.xfer.path) {
     if(session.xfer.p) {
       destroy_pool(session.xfer.p);
-      bzero(&session.xfer,sizeof(session.xfer));
+      memset(&session.xfer, '\0', sizeof(session.xfer));
     }
 
     add_response_err(R_503,"Bad sequence of commands.");
@@ -2859,19 +2862,25 @@ MODRET cmd_rnto(cmd_rec *cmd)
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
   preg = (regex_t*)get_param_ptr(TOPLEVEL_CONF,"PathAllowFilter",FALSE);
 
-  if(preg && regexec(preg,cmd->arg,0,NULL,0) != 0) {
+  if (preg && regexec(preg,cmd->arg,0,NULL,0) != 0) {
     log_debug(DEBUG2, "'%s %s' denied by PathAllowFilter", cmd->argv[0],
       cmd->arg);
-    add_response_err(R_550,"%s: Forbidden filename",cmd->arg);
+    add_response_err(R_550,"%s: Forbidden filename", cmd->arg);
+    destroy_pool(session.xfer.p);
+    memset(&session.xfer, '\0', sizeof(session.xfer));
+
     return ERROR(cmd);
   }
 
   preg = (regex_t*)get_param_ptr(TOPLEVEL_CONF,"PathDenyFilter",FALSE);
 
-  if(preg && regexec(preg,cmd->arg,0,NULL,0) == 0) {
+  if (preg && regexec(preg,cmd->arg,0,NULL,0) == 0) {
     log_debug(DEBUG2, "'%s %s' denied by PathDenyFilter", cmd->argv[0],
       cmd->arg);
-    add_response_err(R_550,"%s: Forbidden filename",cmd->arg);
+    add_response_err(R_550,"%s: Forbidden filename", cmd->arg);
+    destroy_pool(session.xfer.p);
+    memset(&session.xfer, '\0', sizeof(session.xfer));
+
     return ERROR(cmd);
   }
 #endif
@@ -2882,13 +2891,16 @@ MODRET cmd_rnto(cmd_rec *cmd)
      || rename(session.xfer.path,path) == -1) {
     add_response_err(R_550,"rename: %s",strerror(errno));
     destroy_pool(session.xfer.p);
-    bzero(&session.xfer,sizeof(session.xfer));
+    memset(&session.xfer, '\0', sizeof(session.xfer));
+
     return ERROR(cmd);
   }
 
-  add_response(R_250,"rename successful.");
+  add_response(R_250, "rename successful.");
+
   destroy_pool(session.xfer.p);
-  bzero(&session.xfer,sizeof(session.xfer));
+  memset(&session.xfer, '\0', sizeof(session.xfer));
+
   return HANDLED(cmd);
 }
 
@@ -2931,14 +2943,14 @@ MODRET cmd_rnfr(cmd_rec *cmd)
   }
 
   /* We store the path in session.xfer.path */
-  if(session.xfer.p) {
+  if (session.xfer.p) {
     destroy_pool(session.xfer.p);
-    bzero(&session.xfer,sizeof(session.xfer));
+    memset(&session.xfer, '\0', sizeof(session.xfer));
   }
 
   session.xfer.p = make_sub_pool(session.pool);
   session.xfer.path = pstrdup(session.xfer.p,path);
-  add_response(R_350,"File or directory exists, ready for destination name.");
+  add_response(R_350, "File or directory exists, ready for destination name.");
 
   return HANDLED(cmd);
 }
@@ -3237,7 +3249,6 @@ static conftable core_conftable[] = {
   { "AllowUser",		add_allowuser,			NULL },
   { "AnonRequirePassword",	set_anonrequirepassword,	NULL },
   { "AnonymousGroup",		add_anonymousgroup,		NULL },
-  { "AuthAliasOnly",		set_authaliasonly,		NULL },
   { "AuthUsingAlias",		set_authusingalias,		NULL },
   { "Bind",			add_bind,			NULL },
   { "CDPath",			add_cdpath,			NULL },
