@@ -25,7 +25,7 @@
 
 /*
  * Data transfer module for ProFTPD
- * $Id: mod_xfer.c,v 1.57 2001/08/16 19:54:04 flood Exp $
+ * $Id: mod_xfer.c,v 1.65 2002/05/21 20:47:19 castaglia Exp $
  */
 
 /* History Log:
@@ -95,7 +95,7 @@ static void _log_transfer(char direction, char abort_flag) {
   }
 
   log_debug(DEBUG1, "Transfer %s %d bytes in %d.%02d seconds.",
-	    abort_flag == 'c' ? " completed:" : " aborted after",
+	    abort_flag == 'c' ? "completed:" : "aborted after",
 	    session.xfer.total_bytes,end_time.tv_sec,
 	    (end_time.tv_usec / 10000));
 }
@@ -286,64 +286,76 @@ static long _transmit_data(int rate_bps, unsigned long count, off_t offset,
 #endif /* HAVE_SENDFILE */
 }
 
-static void _stor_done() {
+static void _stor_chown(void) {
   struct stat sbuf;
-  
-  fs_close(stor_file,stor_fd);
-  stor_file = NULL;
+  char *xfer_path = NULL;
 
+  if (session.xfer.xfer_type == STOR_HIDDEN)
+    xfer_path = session.xfer.path_hidden;
+  else
+    xfer_path = session.xfer.path;
+ 
   /* session.fsgid defaults to -1, so chown(2) won't chgrp unless specifically
    * requested via GroupOwner
    * jss - 07/04/2001
    */
-  if((session.fsuid != -1) && session.xfer.path) {
-    int err = 0,iserr = 0;
+  if ((session.fsuid != (uid_t) -1) && xfer_path) {
+    int err = 0, iserr = 0;
     
-    fs_stat(session.xfer.path,&sbuf);
+    fs_stat(xfer_path, &sbuf);
     
-    PRIVS_ROOT;
-    if(fs_chown(session.xfer.path,session.fsuid,session.fsgid) == -1) {
+    PRIVS_ROOT
+    if (fs_chown(xfer_path, session.fsuid, session.fsgid) == -1) {
       iserr++;
       err = errno;
     }
-    PRIVS_RELINQUISH;
+    PRIVS_RELINQUISH
+
+    if (iserr) {
+      log_pri(LOG_WARNING, "chown(%s) as root failed: %s", xfer_path,
+        strerror(err));
     
-    if(iserr)
-      log_pri(LOG_WARNING, "chown(%s) as root failed: %s.",
-              session.xfer.path, strerror(err));
-    else {
-      if(session.fsgid != -1)
+    } else {
+      if (session.fsgid != (gid_t) -1)
         log_debug(DEBUG2, "root chown(%s) to uid %lu, gid %lu successful",
-                  session.xfer.path,
+                  xfer_path,
                   (unsigned long)session.fsuid,
                   (unsigned long)session.fsgid);
       else
         log_debug(DEBUG2, "root chown(%s) to uid %lu successful",
-                  session.xfer.path,
+                  xfer_path,
                   (unsigned long)session.fsuid);
       
-      fs_chmod(session.xfer.path,sbuf.st_mode);
+      fs_chmod(xfer_path, sbuf.st_mode);
     }
-  } else if((session.fsgid != -1) && session.xfer.path) {
-    fs_stat(session.xfer.path,&sbuf);
-    if(fs_chown(session.xfer.path,(uid_t)-1,(gid_t)session.fsgid) == -1)
-      log_pri(LOG_WARNING, "chown(%s) failed: %s.",
-              session.xfer.path, strerror(errno));
-    else {
+
+  } else if ((session.fsgid != (gid_t) -1) && xfer_path) {
+    fs_stat(xfer_path, &sbuf);
+
+    if (fs_chown(xfer_path, (uid_t)-1, session.fsgid) == -1) {
+      log_pri(LOG_WARNING, "chown(%s) failed: %s", xfer_path,
+         strerror(errno));
+
+    } else {
       log_debug(DEBUG2, "chown(%s) to gid %lu successful",
-                session.xfer.path,
+                xfer_path,
                 (unsigned long)session.fsgid);
-      fs_chmod(session.xfer.path,sbuf.st_mode);
+      fs_chmod(xfer_path, sbuf.st_mode);
     }
   }
 }
 
-static void _retr_done() {
+static void _stor_done(void) {
+  fs_close(stor_file, stor_fd);
+  stor_file = NULL;
+}
+
+static void _retr_done(void) {
   fs_close(retr_file,retr_fd);
   retr_file = NULL;
 }
 
-static void _stor_abort() {
+static void _stor_abort(void) {
   fs_close(stor_file,stor_fd);
   stor_file = NULL;
 
@@ -359,7 +371,7 @@ static void _stor_abort() {
   _log_transfer('i', 'i');
 }
 
-static void _retr_abort() {
+static void _retr_abort(void) {
   /* Isn't necessary to send anything here, just cleanup */
   fs_close(retr_file,retr_fd);
   retr_file = NULL;
@@ -367,7 +379,7 @@ static void _retr_abort() {
 }
 
 /* Exit handler, call abort functions if a transfer is in progress. */
-static void _xfer_exit() {
+static void _xfer_exit(void) {
   if(session.flags & SF_XFER) {
     if(session.xfer.direction == IO_READ) /* stor */
       _stor_abort();
@@ -651,18 +663,26 @@ MODRET cmd_stor(cmd_rec *cmd)
   if(!stor_file) {
     add_response_err(R_550,"%s: %s",cmd->arg,strerror(errno));
     return ERROR(cmd);
-  } else {
-    /* perform the actual transfer now */
-    data_init(cmd->arg,IO_READ);
 
-    session.xfer.path = pstrdup(session.xfer.p,dir);
-    if(session.xfer.xfer_type == STOR_HIDDEN)
-      session.xfer.path_hidden = pstrdup(session.xfer.p, p_hidden->value.str_val);
+  } else {
+
+    /* perform the actual transfer now */
+    data_init(cmd->arg, IO_READ);
+
+    session.xfer.path = pstrdup(session.xfer.p, dir);
+    if (session.xfer.xfer_type == STOR_HIDDEN)
+      session.xfer.path_hidden = pstrdup(session.xfer.p,
+        p_hidden->value.str_val);
     else
       session.xfer.path_hidden = NULL;
       
     session.xfer.file_size = respos;
 
+    /* first, make sure the uploaded file has the requested ownership
+     * -tj, 2001-10-18
+     */
+    _stor_chown();
+
     if(data_open(cmd->arg,NULL,IO_READ,0) < 0) {
       _stor_abort();
       data_abort(0,TRUE);
@@ -848,7 +868,7 @@ MODRET cmd_retr(cmd_rec *cmd)
   dir = p->value.str_val;
   retr_file = fs_open(dir,O_RDONLY,&retr_fd);
 
-  if(session.restart_pos) {
+  if(retr_file && session.restart_pos) {
     if(fs_lseek(retr_file,retr_fd,session.restart_pos,SEEK_SET) == -1) {
       int _errno = errno;
       fs_close(retr_file,retr_fd);
@@ -947,6 +967,7 @@ MODRET cmd_abor(cmd_rec *cmd)
 
   add_response(R_226,"Abort successful");
   data_abort(0,FALSE);
+  data_reset();
   data_cleanup();
   
   return HANDLED(cmd);
@@ -1082,6 +1103,14 @@ cmd_stou(cmd_rec *cmd)
 	return HANDLED(cmd);
 }
 
+MODRET xfer_err_cleanup(cmd_rec *cmd) {
+  if (session.xfer.p)
+    destroy_pool(session.xfer.p);
+
+  memset(&session.xfer, '\0', sizeof(session.xfer));
+  return DECLINED(cmd);
+}
+
 MODRET log_stor(cmd_rec *cmd)
 {
   _log_transfer('i', 'c');
@@ -1119,7 +1148,7 @@ static int _noxfer_timeout(CALLBACK_FRAME)
   return 0;
 }
 
-int xfer_init_child()
+int xfer_init_child(void)
 {
   /* Setup TimeoutNoXfer timer */
   if(TimeoutNoXfer)
@@ -1200,8 +1229,12 @@ static cmdtable xfer_commands[] = {
   { PRE_CMD, C_STOR,	G_WRITE, pre_cmd_stor,	TRUE,	FALSE },
   { CMD,     C_STOR,	G_WRITE, cmd_stor,	TRUE,	FALSE, CL_WRITE },
   { LOG_CMD, C_STOR,    G_NONE,	 log_stor,	FALSE,  FALSE },
+  { LOG_CMD_ERR, C_STOR,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
+  { LOG_CMD_ERR, C_RETR,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
   { CMD,     C_STOU,	G_WRITE, cmd_stou,	TRUE,	FALSE, CL_WRITE },
   { PRE_CMD, C_APPE,	G_WRITE, pre_cmd_appe,	TRUE,	FALSE },
+  { LOG_CMD_ERR, C_STOU,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
+  { LOG_CMD_ERR, C_APPE,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
   { CMD,     C_APPE,	G_WRITE, cmd_stor,	TRUE,	FALSE, CL_WRITE },
   { LOG_CMD, C_APPE,	G_NONE,  log_stor,	FALSE,  FALSE },
   { CMD,     C_ABOR,	G_NONE,	 cmd_abor,	TRUE,	TRUE,  CL_MISC  },
