@@ -25,7 +25,7 @@
  
 /*
  * Data connection management functions
- * $Id: data.c,v 1.26 2001/06/19 18:51:51 flood Exp $
+ * $Id: data.c,v 1.30 2002/05/21 20:47:22 castaglia Exp $
  */
 
 #include "conf.h"
@@ -128,7 +128,7 @@ static int _xlate_ascii_write(char **buf, int *bufsize, int *adjlen) {
 static void _data_new_xfer(char *filename, int direction) {
   if(session.xfer.p) {
     destroy_pool(session.xfer.p);
-    bzero(&session.xfer,sizeof(session.xfer));
+    memset(&session.xfer,0,sizeof(session.xfer));
   }
   
   session.xfer.p = make_sub_pool(session.pool);
@@ -259,7 +259,7 @@ static int _data_active_open(char *reason, unsigned long size) {
   return -1;
 }
 
-void data_reset() {
+void data_reset(void) {
   if(session.d && session.d->pool)
     destroy_pool(session.d->pool);
   session.d = NULL;
@@ -367,15 +367,6 @@ void data_close(int quiet) {
     session.d = NULL;
   }
   
-#if 0
-  if(session.xfer.p)
-    destroy_pool(session.xfer.p);
-  
-  bzero(&session.xfer,sizeof(session.xfer));
-#endif
-  
-  session.data_port = session.c->remote_port - 1;
-  
   /* aborts no longer necessary */
   signal(SIGURG,SIG_IGN);
   
@@ -402,13 +393,27 @@ void data_close(int quiet) {
  * flags in any case.  session flags will end up have SF_POST_ABORT
  * set if the OOB byte won the race.
  */
-void data_cleanup() {
+void data_cleanup(void) {
+  /* sanity check */
+  if(session.d) {
+    inet_close(session.pool,session.d);
+    session.d = NULL;
+  }
+
   if(session.xfer.p)
     destroy_pool(session.xfer.p);
   
-  bzero(&session.xfer,sizeof(session.xfer));
+  memset(&session.xfer,0,sizeof(session.xfer));
+  
+  session.data_port = session.c->remote_port - 1;
 }
 
+/* In order to avoid clearing the transfer counters in session.xfer, we don't
+ * clear session.xfer here, it should be handled by the appropriate
+ * LOG_CMD/LOG_CMD_ERR handler calling data_cleanup().
+ * jss - 12/13/2001
+ */
+
 void data_abort(int err, int quiet) {
   int true_abort = XFER_ABORTED;
   
@@ -418,17 +423,10 @@ void data_abort(int err, int quiet) {
     session.d = NULL;
   }
   
-  if(session.xfer.p)
-    destroy_pool(session.xfer.p);
-  
-  bzero(&session.xfer,sizeof(session.xfer));
-  
-  session.data_port = session.c->remote_port - 1;
-  
-  if(TimeoutNoXfer)
+  if (TimeoutNoXfer)
     reset_timer(TIMER_NOXFER,ANY_MODULE);
   
-  if(TimeoutStalled)
+  if (TimeoutStalled)
     remove_timer(TIMER_STALLED,ANY_MODULE);
   
   session.flags &= (SF_ALL^SF_PASSIVE);
@@ -651,7 +649,7 @@ int data_xfer(char *cl_buf, int cl_size) {
 	      adjlen += (buflen - cl_size);
 	      buflen = cl_size;
 	    }
-  	    bcopy(buf,cl_buf,buflen);
+  	    memcpy(cl_buf,buf,buflen);
 	
 	    /* copy whatever remains at the end of session.xfer.buf to the
 	     * head of the buffer and adjust buf accordingly
@@ -661,7 +659,7 @@ int data_xfer(char *cl_buf, int cl_size) {
 	     */
 	
 	    if(adjlen > 0)
-	      bcopy(buf+buflen,buf,adjlen);
+	      memcpy(buf,buf+buflen,adjlen);
 
 	    /* store everything back in session.xfer */
 	    session.xfer.buflen = adjlen;
@@ -699,7 +697,7 @@ int data_xfer(char *cl_buf, int cl_size) {
 	size = TUNABLE_BUFFER_SIZE;
       
       o_size = size;
-      bcopy(cl_buf,buf,size);
+      memcpy(buf,cl_buf,size);
       while(size) {
 	wb = buf; wsize = size; adjlen = 0;
 	
@@ -716,7 +714,7 @@ int data_xfer(char *cl_buf, int cl_size) {
 	size -= (wsize - adjlen);
 	if(size) {
 	  wb = buf + (wsize - adjlen);
-	  bcopy(wb,buf,size);
+	  memcpy(buf,wb,size);
 	}
       }
       
@@ -767,7 +765,27 @@ pr_sendfile_t data_sendfile(int retr_fd, off_t *offset, size_t count) {
      *
      * ssize_t sendfile(int out_fd, int in_fd, off_t *offset, size_t count)
      */
-    if((len = sendfile(session.d->outf->fd, retr_fd, offset, count)) == -1) {
+    len = sendfile(session.d->outf->fd, retr_fd, offset, count);
+    
+    if(len != -1 && len < count) {
+      /* under linux semantics, this occurs when a signal has interrupted
+       * sendfile().
+       */
+
+      count -= len;
+	
+      if(TimeoutStalled)
+        reset_timer(TIMER_STALLED, ANY_MODULE);
+	
+      if(TimeoutIdle)
+	reset_timer(TIMER_IDLE, ANY_MODULE);
+	
+      session.xfer.total_bytes += len;
+      session.total_bytes += len;
+      total += len;
+        
+      continue;
+    } else if(len == -1) {
       /* Linux updates offset on error, not len like BSD, fix up so
        * BSD-based code works.
        */
