@@ -25,7 +25,7 @@
 
 /* Read configuration file(s), and manage server/configuration
  * structures.
- * $Id: dirtree.c,v 1.40 2001/10/19 15:27:58 flood Exp $
+ * $Id: dirtree.c,v 1.53 2002/05/21 20:47:22 castaglia Exp $
  */
 
 /* History:
@@ -56,6 +56,9 @@ int TimeoutNoXfer = TUNABLE_TIMEOUTNOXFER;
 int TimeoutStalled = TUNABLE_TIMEOUTSTALLED;
 char MultilineRFC2228 = 0;
 
+/* from src/pool.c */
+extern pool *global_config_pool;
+
 /* Used by find_config_* */
 xaset_t *find_config_top = NULL;
 
@@ -149,12 +152,12 @@ int dir_check_hidden(const char *path) {
   return FALSE;	
 }
 
-void kludge_disable_umask()
+void kludge_disable_umask(void)
 {
   _kludge_disable_umask = 1;
 }
 
-void kludge_enable_umask()
+void kludge_enable_umask(void)
 {
   _kludge_disable_umask = 0;
 }
@@ -281,7 +284,7 @@ void init_dyn_stacks(pool *p,config_rec *top)
   *conf.curconfig = top;
 }
 
-void init_conf_stacks()
+void init_conf_stacks(void)
 {
   pool *pool = make_sub_pool(permanent_pool);
 
@@ -294,15 +297,15 @@ void init_conf_stacks()
   *conf.curconfig = NULL;
 }
 
-void free_dyn_stacks()
+void free_dyn_stacks(void)
 {
-  bzero(&conf,sizeof(conf));
+  memset(&conf, 0, sizeof(conf));
 }
 
-void free_conf_stacks()
+void free_conf_stacks(void)
 {
   destroy_pool(conf.tpool);
-  bzero(&conf, sizeof(conf));
+  memset(&conf, 0, sizeof(conf));
 }
 
 /* Used by modules to start/end configuration sections */
@@ -316,6 +319,7 @@ server_rec *start_new_server(const char *addr)
 
   s = (server_rec*)pcalloc(p,sizeof(server_rec));
   s->pool = p;
+  s->config_type = CONF_VIRTUAL;
   
   /* Have to make sure it ends up on the end of the chain,
    * otherwise main_server becomes useless.
@@ -334,7 +338,7 @@ server_rec *start_new_server(const char *addr)
   return s;
 }
 
-server_rec *end_new_server()
+server_rec *end_new_server(void)
 {
   if(!*conf.curserver)
     return NULL;
@@ -352,34 +356,49 @@ server_rec *end_new_server()
 
 /* Starts a sub-configuration */
   
-config_rec *start_sub_config(const char *name)
-{
-  config_rec *c,*parent = *conf.curconfig;
-  pool *p;
-  xaset_t **set;
+config_rec *start_sub_config(const char *name) {
+  config_rec *c = NULL, *parent = *conf.curconfig;
+  pool *c_pool = NULL, *parent_pool = NULL;
+  xaset_t **set = NULL;
 
-  if(parent) {
-    p = make_sub_pool(parent->pool);
+  if (parent) {
+    parent_pool = parent->pool;
     set = &parent->subset;
+
   } else {
-    p = make_sub_pool((*conf.curserver)->pool);
+    parent_pool = (*conf.curserver)->pool;
     set = &(*conf.curserver)->conf;
   }
 
-  c = (config_rec*)pcalloc(p,sizeof(config_rec));
+  /* allocate a sub-pool for this config_rec.  Note: special
+   * exception for <Global> configs -- the parent pool is global_config_pool
+   * (a pool just for this context), not the pool of the parent server.  This
+   * keeps <Global> config recs from being freed prematurely, and helps to
+   * avoid memory leaks.
+   */
+  if (!strcmp(name, "<Global>")) {
+    if (!global_config_pool)
+      global_config_pool = make_named_sub_pool(permanent_pool,
+        "<Global> configs");
+    parent_pool = global_config_pool;
+  }
 
-  if(!*set)
-    *set = xaset_create(p,NULL);
+  c_pool = make_sub_pool(parent_pool);
+  c = (config_rec *) pcalloc(c_pool, sizeof(config_rec));
 
-  xaset_insert(*set,(xasetmember_t*)c);
+  if (!*set)
+    *set = xaset_create(c_pool, NULL);
+
+  xaset_insert(*set, (xasetmember_t*)c);
   
-  c->pool = p;
+  c->pool = c_pool;
   c->set = *set;
   c->parent = parent;
-  if(name)
-    c->name = pstrdup(p,name);
 
-  if(parent && (parent->config_type == CONF_DYNDIR))
+  if (name)
+    c->name = pstrdup(c->pool, name);
+
+  if (parent && (parent->config_type == CONF_DYNDIR))
     c->flags |= CF_DYNAMIC;
 
   /* Now insert another level onto the stack */
@@ -394,7 +413,7 @@ config_rec *start_sub_config(const char *name)
 }
 
 /* Pop one level off the stack */
-config_rec *end_sub_config()
+config_rec *end_sub_config(void)
 {
   if(conf.curconfig == (config_rec**)conf.cstack->elts) {
     if(*conf.curconfig)
@@ -450,17 +469,17 @@ config_rec *add_config_set(xaset_t **set,const char *name)
 }
 
 /* Adds a config_rec on the current "level" */
-config_rec *add_config(const char *name)
-{
+config_rec *add_config(const char *name) {
   server_rec *s = *conf.curserver;
-  config_rec *parent,*c = *conf.curconfig;
-  pool *p;
-  xaset_t **set;
+  config_rec *parent = NULL, *c = *conf.curconfig;
+  pool *p = NULL;
+  xaset_t **set = NULL;
 
-  if(c) {
+  if (c) {
     parent = c;
     p = c->pool;
     set = &c->subset;
+
   } else {
     parent = NULL;
     
@@ -468,13 +487,14 @@ config_rec *add_config(const char *name)
       p = make_sub_pool(s->pool);
     else
       p = ((config_rec*)s->conf->xas_list)->pool;
+
     set = &s->conf;
   }
 
-  if(!*set)
-    *set = xaset_create(p,NULL);
+  if (!*set)
+    *set = xaset_create(p, NULL);
 
-  c = add_config_set(set,name);
+  c = add_config_set(set, name);
   c->parent = parent;
 
   return c;
@@ -525,10 +545,13 @@ int group_expression(char **expr)
       grp++;
     }
 
-    for(cnt = session.groups->nelts-1; cnt >= 0; cnt--)
-      if(strcmp(*(((char**)session.groups->elts)+cnt),grp) == 0) {
-        found = !found; break;
-      }
+    if(session.group && strcmp(session.group,grp) == 0)
+      found = !found;
+    else if (session.groups)
+      for(cnt = session.groups->nelts-1; cnt >= 0; cnt--)
+        if(strcmp(*(((char**)session.groups->elts)+cnt),grp) == 0) {
+          found = !found; break;
+        }
 
     if(!found) {
       expr = NULL;
@@ -707,12 +730,11 @@ int dir_get_param(pool *pp,char *path,char *param)
   return -1;
 }
 
-static int _dir_check_op(pool *p,xaset_t *c,int op,
-                         uid_t uid,gid_t gid,mode_t mode)
-{
+static int _dir_check_op(pool *p, xaset_t *c, int op, uid_t uid, gid_t gid,
+    mode_t mode) {
   int i, res = 1, user_perms = 0;
-  uid_t u;
-  gid_t *gidp = NULL, g;
+  uid_t *u = NULL;
+  gid_t *g = NULL, *gidp = NULL;
 
   if(!c)
     return 1;				/* Default is to allow */
@@ -738,10 +760,10 @@ static int _dir_check_op(pool *p,xaset_t *c,int op,
     /* loop through the user's auxiliary groups, checking if these
      * memberships match that of the file
      */
-    for (i = session.gids->nelts, gidp = (int *) session.gids->elts;
+    for (i = session.gids->nelts, gidp = (gid_t *) session.gids->elts;
          i; i--, gidp++) {
 
-			/* matched an auxiliary GID against the file GID
+      /* matched an auxiliary GID against the file GID
        */
       if (*gidp == gid) {
         found_gid_match = TRUE;
@@ -759,22 +781,22 @@ static int _dir_check_op(pool *p,xaset_t *c,int op,
 
   switch(op) {
   case OP_HIDE:
-    u = get_param_int(c,"HideUser",FALSE);
+    u = (uid_t *) get_param_ptr(c, "HideUser", FALSE);
 
-    while(u != -1 && (u != uid || u == session.uid))
-      u = get_param_int_next("HideUser",FALSE);
+    while (u && *u != (uid_t) -1 && (*u != uid || *u == session.uid))
+      u = (uid_t *) get_param_ptr_next("HideUser", FALSE);
 
-    if(u == uid) {
+    if (u && *u == uid) {
       res = 0;
       break;
     }
 
-    g = get_param_int(c,"HideGroup",FALSE);
+    g = (gid_t *) get_param_ptr(c, "HideGroup", FALSE);
 
-    while(g != -1 && (g != gid || g == session.gid))
-      g = get_param_int_next("HideGroup",FALSE);
+    while (g && *g != (gid_t) -1 && (*g != gid || *g == session.gid))
+      g = (gid_t *) get_param_ptr_next("HideGroup", FALSE);
 
-    if(g == gid) {
+    if (g && *g == gid) {
       res = 0;
       break;
     }
@@ -1370,23 +1392,24 @@ void build_dyn_config(pool *p,char *_path, struct stat *_sbuf, int recurse)
         newd->argv = pcalloc(newd->pool,2*sizeof(void*));
 	newd->parent = d;
         d = newd;
-      } else if(d->subset && d->subset->xas_list &&
-                strcmp(d->name,fullpath) == 0 && 
+      } else if(strcmp(d->name,fullpath) == 0 && 
                 (isfile == -1 || sbuf.st_mtime > (time_t)d->argv[0])) {
         set = (d->parent ? &d->parent->subset :
                &main_server->conf);
 
-        /* remove all old dynamic entries */
-        for(newd = (config_rec*)d->subset->xas_list; newd; newd=dnext) {
-          dnext = newd->next;
+	if (d->subset && d->subset->xas_list) {
+       	  /* remove all old dynamic entries */
+          for(newd = (config_rec*)d->subset->xas_list; newd; newd=dnext) {
+	    dnext = newd->next;
 
-          if(newd->flags & CF_DYNAMIC) {
-            xaset_remove(d->subset,(xasetmember_t*)newd);
-            removed++;
+            if(newd->flags & CF_DYNAMIC) {
+              xaset_remove(d->subset,(xasetmember_t*)newd);
+              removed++;
+            }
           }
-        }
+	}
 
-        if(!d->subset->xas_list) {
+        if(d->subset && !d->subset->xas_list) {
           destroy_pool(d->subset->mempool);
           d->subset = NULL;
           d->argv[0] = NULL;
@@ -1500,7 +1523,7 @@ int dir_check_full(pool *pp, char *cmd, char *group, char *path, int *hidden)
   config_rec *c;
   struct stat sbuf;
   pool *p;
-  mode_t _umask = -1;
+  mode_t _umask = (mode_t) -1;
   int res = 1, isfile;
   int op_hidden = FALSE;
 
@@ -1528,7 +1551,7 @@ int dir_check_full(pool *pp, char *cmd, char *group, char *path, int *hidden)
 
   /* Check and build all appropriate dynamic configuration entries */
   if((isfile = fs_stat(path, &sbuf)) == -1)
-    bzero(&sbuf, sizeof(sbuf));
+    memset(&sbuf,0,sizeof(sbuf));
   
   build_dyn_config(p,path,&sbuf,1);
   
@@ -1537,8 +1560,6 @@ int dir_check_full(pool *pp, char *cmd, char *group, char *path, int *hidden)
   if(!c && session.anon_config)
     c = session.anon_config;
 
-  _umask = (mode_t) -1;
-  
   if(!_kludge_disable_umask) {
     /* Check for a directory Umask.
      */
@@ -1567,7 +1588,9 @@ int dir_check_full(pool *pp, char *cmd, char *group, char *path, int *hidden)
     }
   }
   
-  session.fsuid = session.fsgid = -1;
+  session.fsuid = (uid_t) -1;
+  session.fsgid = (gid_t) -1;
+
   if((owner = get_param_ptr(CURRENT_CONF,"UserOwner",FALSE)) != NULL) {
     /* attempt chown on all new files */
     struct passwd *pw;
@@ -1638,7 +1661,7 @@ int dir_check(pool *pp, char *cmd, char *group, char *path, int *hidden)
   config_rec *c;
   struct stat sbuf;
   pool *p;
-  mode_t _umask = -1;
+  mode_t _umask = (mode_t) -1;
   int res = 1, isfile;
   int op_hidden = FALSE;
 
@@ -1659,7 +1682,7 @@ int dir_check(pool *pp, char *cmd, char *group, char *path, int *hidden)
 
   /* Check and build all appropriate dynamic configuration entries */
   if((isfile = fs_stat(path, &sbuf)) == -1)
-    bzero(&sbuf, sizeof(sbuf));
+    memset(&sbuf,0,sizeof(sbuf));
 
   build_dyn_config(p, path, &sbuf, 0);
 
@@ -1668,8 +1691,6 @@ int dir_check(pool *pp, char *cmd, char *group, char *path, int *hidden)
   if(!c && session.anon_config)
     c = session.anon_config;
 
-  _umask = -1;
-  
   if(!_kludge_disable_umask) {
     /* Check for a directory Umask.
      */
@@ -1698,7 +1719,9 @@ int dir_check(pool *pp, char *cmd, char *group, char *path, int *hidden)
     }
   }
 
-  session.fsuid = session.fsgid = -1;
+  session.fsuid = (uid_t) -1;
+  session.fsgid = (gid_t) -1;
+
   if((owner = get_param_ptr(CURRENT_CONF,"UserOwner",FALSE)) != NULL) {
     /* attempt chown on all new files */
     struct passwd *pw;
@@ -1917,16 +1940,18 @@ static void _mergedown(xaset_t *s,int dynamic)
   if(!s || !s->xas_list)
     return;
 
-  for(c = (config_rec*)s->xas_list; c; c=c->next)
-    if(c->flags & CF_MERGEDOWN)
+  for (c = (config_rec*)s->xas_list; c; c=c->next)
+    if ((c->flags & CF_MERGEDOWN) ||
+        (c->flags & CF_MERGEDOWN_MULTI))
       for(dest = (config_rec*)s->xas_list; dest; dest=dest->next)
         if(dest->config_type == CONF_ANON ||
            dest->config_type == CONF_DIR) {
+
           /* If an option of the same name/type is found in the
            * next level down, it overrides, so we don't merge.
            */
-          if(find_config(dest->subset,c->config_type,
-                         c->name,FALSE))
+          if ((c->flags & CF_MERGEDOWN) &&
+              find_config(dest->subset, c->config_type, c->name, FALSE))
             continue;
 
           if(!dest->subset)
@@ -2053,7 +2078,7 @@ void _copy_global_to_all(xaset_t *set)
       _copy_recur(&s->conf,s->pool,c,NULL);
 }
 
-void fixup_globals()
+void fixup_globals(void)
 {
   server_rec *s,*smain;
   config_rec *c,*cnext;
@@ -2340,25 +2365,24 @@ config_rec *add_config_param_set(xaset_t **set,const char *name,int num,...)
   return c;
 }
 
-config_rec *add_config_param_str(const char *name, int num, ...)
-{
+config_rec *add_config_param_str(const char *name, int num, ...) {
   config_rec *c = add_config(name);
-  char *arg;
-  void **argv;
+  char *arg = NULL;
+  void **argv = NULL;
   va_list ap;
 
-  if(c) {
+  if (c) {
     c->config_type = CONF_PARAM;
     c->argc = num;
-    c->argv = pcalloc(c->pool,(num+1) * sizeof(char*));
+    c->argv = pcalloc(c->pool, (num+1) * sizeof(char*));
 
     argv = c->argv;
-    va_start(ap,num);
+    va_start(ap, num);
 
     while(num-- > 0) {
-      arg = va_arg(ap,char*);
-      if(arg)
-        *argv++ = pstrdup(permanent_pool,arg);
+      arg = va_arg(ap, char*);
+      if (arg)
+        *argv++ = pstrdup(c->pool, arg);
       else
         *argv++ = NULL;
     }
@@ -2448,7 +2472,7 @@ int parse_config_file(const char *fname)
  * otherwise fill in defaults where applicable
  */
 
-void fixup_servers()
+void fixup_servers(void)
 {
   config_rec *c;
   server_rec *s;
@@ -2479,7 +2503,11 @@ void fixup_servers()
     if(!s->tcp_swin)
       s->tcp_swin = TUNABLE_DEFAULT_SWIN;
 
-    s->ipaddr = inet_getaddr(s->pool,s->ServerAddress);
+    if ((s->ipaddr = inet_getaddr(s->pool, s->ServerAddress)) == NULL) {
+      log_pri(LOG_ERR,"Fatal: unable to determine IP address of '%s'.",
+        s->ServerAddress);
+      exit(1);
+    }
 
     if ((c = find_config(s->conf, CONF_PARAM, "MasqueradeAddress",
         FALSE)) != NULL) {
@@ -2488,13 +2516,16 @@ void fixup_servers()
         inet_ascii(s->pool, (p_in_addr_t *) c->argv[0]));
     }
 
-    if(!s->ipaddr) {
-      log_pri(LOG_ERR,"Fatal: unable to determine IP address of `%s'.",
-                      s->ServerAddress);
-      exit(1);
+    /* honor the DefaultServer directive only if SocketBindTight is not
+     * in effect.
+     */
+    if (get_param_int(s->conf, "DefaultServer", FALSE) == 1) {
+      if (!SocketBindTight)
+        s->ipaddr->s_addr = 0;
+      else
+        log_pri(LOG_NOTICE,
+          "SocketBindTight in effect, ignoring DefaultServer");
     }
-    if(get_param_int(s->conf,"DefaultServer",FALSE) == 1)
-      s->ipaddr->s_addr = 0;
 
     fixup_dirs(s,0);
   }
@@ -2502,10 +2533,15 @@ void fixup_servers()
   clear_inet_pool();
 }
 
-void init_config()
-{
+void init_config(void) {
   pool *pool = make_sub_pool(permanent_pool);
 
+  /* Make sure global_config_pool is destroyed */
+  if (global_config_pool) {
+    destroy_pool(global_config_pool);
+    global_config_pool = NULL;
+  }
+
   servers = xaset_create(pool,NULL);
 
   pool = make_sub_pool(permanent_pool);
@@ -2522,34 +2558,38 @@ void init_config()
 /* These functions are used by modules to help parse configuration.
  */
 
-int check_conf(cmd_rec *cmd, int allowed)
-{
-  int x;
+unsigned char check_conf(cmd_rec *cmd, int allowed) {
+  int ctxt = (cmd->config && cmd->config->config_type != CONF_PARAM ?
+     cmd->config->config_type : cmd->server->config_type ?
+     cmd->server->config_type : CONF_ROOT);
 
-  x = (cmd->config && cmd->config->config_type != CONF_PARAM ?
-       cmd->config->config_type : CONF_ROOT);
+  if (ctxt & allowed)
+    return TRUE;
 
-  return (x & allowed);
+  /* default */
+  return FALSE;
 }
 
-char *get_section_name(cmd_rec *cmd)
-{
+char *get_context_name(cmd_rec *cmd) {
   static char cbuf[20];
 
-  if(!cmd->config || cmd->config->config_type == CONF_PARAM)
-    return "top level";
+  if (!cmd->config || cmd->config->config_type == CONF_PARAM) {
+    if (cmd->server->config_type == CONF_VIRTUAL)
+      return "<VirtualHost>"; 
+    else
+      return "server config";
+  }    
 
   memset(cbuf,'\0',sizeof(cbuf));
   switch(cmd->config->config_type) {
-  case CONF_ROOT: return "root";
   case CONF_DIR: return "<Directory>";
   case CONF_ANON: return "<Anonymous>";
   case CONF_LIMIT: return "<Limit>";
-  case CONF_VIRTUAL: return "<VirtualHost>";
   case CONF_DYNDIR: return ".ftpaccess";
   case CONF_GLOBAL: return "<Global>";
   case CONF_USERDATA: return "user data";
   default:
+    /* in 1.3/2.0, should dispatch to modules here */
   snprintf(cbuf, sizeof(cbuf), "%d", cmd->config->config_type);
   return cbuf;
   }
