@@ -62,7 +62,7 @@ static void _inet_pool_cleanup(void *ignore)
 /* Create the private inet pool
  */
 
-static void _create_inet_pool()
+static void _create_inet_pool(void)
 {
   inet_pool = make_sub_pool(permanent_pool);
 
@@ -78,7 +78,7 @@ static void _create_inet_pool()
  * to free up memory.
  */
 
-void clear_inet_pool()
+void clear_inet_pool(void)
 {
   if(!inet_pool)		/* Sanity check */
     return;
@@ -336,13 +336,13 @@ int inet_prebind_socket(pool *p, p_in_addr_t *bind_addr, int port)
 #ifdef SOLARIS2
   if(port != INPORT_ANY && port < 1024) {
     block_signals();
-    PRIVS_ROOT;
+    PRIVS_ROOT
   }
 #endif
   s = socket(AF_INET, SOCK_STREAM, 0);
 #ifdef SOLARIS2
   if(port != INPORT_ANY && port < 1024) {
-    PRIVS_RELINQUISH;
+    PRIVS_RELINQUISH
     unblock_signals();
   }
 #endif
@@ -361,7 +361,7 @@ int inet_prebind_socket(pool *p, p_in_addr_t *bind_addr, int port)
   
   if(port != INPORT_ANY && port < 1024) {
     block_signals();
-    PRIVS_ROOT;
+    PRIVS_ROOT
   }
 
   for(tries = 1; tries < 10; tries++) {
@@ -370,20 +370,20 @@ int inet_prebind_socket(pool *p, p_in_addr_t *bind_addr, int port)
     if(errno != EADDRINUSE) break;
 
     if(port != INPORT_ANY && port < 1024) {
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
       unblock_signals();
     }
     timer_sleep(tries);
     if(port != INPORT_ANY && port < 1024) {
       block_signals();
-      PRIVS_ROOT;
+      PRIVS_ROOT
     }
   }
 
   save_errno = errno;
 
   if(port != INPORT_ANY && port < 1024) {
-    PRIVS_RELINQUISH;
+    PRIVS_RELINQUISH
     unblock_signals();
   }
 
@@ -449,13 +449,13 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
      */
     
 #if defined(SOLARIS2) || defined(FREEBSD2) || defined(FREEBSD3) || \
-    defined(FREEBSD4) || defined(OPENBSD2) || defined(__NetBSD__) || \
+    defined(FREEBSD4) || defined(__OpenBSD__) || defined(__NetBSD__) || \
     defined(SYSV4_2MP)
 # ifdef SOLARIS2
     if(port != INPORT_ANY && port < 1024) {
 # endif
       block_signals();
-      PRIVS_ROOT;
+      PRIVS_ROOT
 # ifdef SOLARIS2
     }
 # endif
@@ -464,12 +464,12 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
     fd = socket(AF_INET, SOCK_STREAM, tcp_proto);
 
 #if defined(SOLARIS2) || defined(FREEBSD2) || defined(FREEBSD3) || \
-    defined(FREEBSD4) || defined(OPENBSD2) || defined(__NetBSD__) || \
+    defined(FREEBSD4) || defined(__OpenBSD__) || defined(__NetBSD__) || \
     defined(SYSV4_2MP)
 # ifdef SOLARIS2
     if(port != INPORT_ANY && port < 1024) {
 # endif
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
       unblock_signals();
 # ifdef SOLARIS2
     }
@@ -497,7 +497,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
     servaddr.sin_family = AF_INET;
     
     if(bind_addr)
-      bcopy(bind_addr, &servaddr.sin_addr, sizeof(servaddr.sin_addr));
+      memcpy(&servaddr.sin_addr, bind_addr, sizeof(servaddr.sin_addr));
     else
       servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
     
@@ -505,7 +505,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
     
     if(port != INPORT_ANY && port < 1024) {
       block_signals();
-      PRIVS_ROOT;
+      PRIVS_ROOT
     }
     
     /* According to one expert, the very nature of the FTP protocol,
@@ -529,7 +529,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
         break;
       
       if(port != INPORT_ANY && port < 1024) {
-        PRIVS_RELINQUISH;
+        PRIVS_RELINQUISH
         unblock_signals();
       }
       
@@ -537,13 +537,13 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
       
       if(port != INPORT_ANY && port < 1024) {
         block_signals();
-        PRIVS_ROOT;
+        PRIVS_ROOT
       }
     }
     
     if(res == -1) {
       if(port != INPORT_ANY && port < 1024) {
-        PRIVS_RELINQUISH;
+        PRIVS_RELINQUISH
         unblock_signals();
       }
 
@@ -561,7 +561,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
     }
     
     if(port != INPORT_ANY && port < 1024) {
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
       unblock_signals();
     }
     
@@ -924,7 +924,10 @@ int inet_connect(pool *pool, conn_t *c, p_in_addr_t *addr, int port)
   }
 
   c->mode = CM_OPEN;
-  inet_get_conn_info(c,c->listen_fd);
+
+  if (inet_get_conn_info(c, c->listen_fd) < 0)
+    return -1;
+
   inet_setblock(c->pool,c);
   return 1;
 }
@@ -956,7 +959,10 @@ int inet_connect_nowait(pool *pool, conn_t *c, p_in_addr_t *addr, int port)
   }
 
   c->mode = CM_OPEN;
-  inet_get_conn_info(c,c->listen_fd);
+
+  if (inet_get_conn_info(c, c->listen_fd) < 0)
+    return -1;
+
   inet_setblock(c->pool,c);
   return 1;
 }
@@ -1040,34 +1046,35 @@ conn_t *inet_accept(pool *pool, conn_t *d, conn_t *c, int rfd, int wfd,
   return res;
 }
 
-int inet_get_conn_info(conn_t *c, int fd)
-{
+int inet_get_conn_info(conn_t *c, int fd) {
   static struct sockaddr_in servaddr;
   int len = sizeof(servaddr);
   
   /* Sanity check.
    */
-  if(fd < 0)
-    return -1;
-  
-  if(getsockname(fd,(struct sockaddr*)&servaddr,&len) != -1) {
-    if(!c->local_ipaddr)
-      c->local_ipaddr = (p_in_addr_t*)pcalloc(c->pool,sizeof(p_in_addr_t));     
-    *c->local_ipaddr = servaddr.sin_addr;
-    c->local_port = ntohs(servaddr.sin_port);
-  } else {
+  if (fd < 0) {
+    errno = EBADF;
     return -1;
   }
+ 
+  if (getsockname(fd, (struct sockaddr *) &servaddr, &len) != -1) {
+    if (!c->local_ipaddr)
+      c->local_ipaddr = (p_in_addr_t *) pcalloc(c->pool, sizeof(p_in_addr_t));     
+    *c->local_ipaddr = servaddr.sin_addr;
+    c->local_port = ntohs(servaddr.sin_port);
+
+  } else
+    return -1;
   
   len = sizeof(servaddr);
 
-  if(getpeername(fd,(struct sockaddr*)&servaddr,&len) != -1) {
-    c->remote_ipaddr = (p_in_addr_t*)pcalloc(c->pool,sizeof(p_in_addr_t));
+  if (getpeername(fd, (struct sockaddr *) &servaddr, &len) != -1) {
+    c->remote_ipaddr = (p_in_addr_t *) pcalloc(c->pool, sizeof(p_in_addr_t));
     *c->remote_ipaddr = servaddr.sin_addr;
     c->remote_port = ntohs(servaddr.sin_port);
-  } else {
+
+  } else
     return -1;
-  }
 
   return 0;
 }
@@ -1106,8 +1113,8 @@ conn_t *inet_associate(pool *pool, conn_t *c, p_in_addr_t *addr,
   res->outf = outf;
   res->mode = CM_OPEN;
 
-
-  inet_get_conn_info(res,wfd);
+  if (inet_get_conn_info(res, wfd) < 0)
+    return NULL;
 
   /* Get the remote address */
 
@@ -1148,48 +1155,59 @@ conn_t *inet_associate(pool *pool, conn_t *c, p_in_addr_t *addr,
 conn_t *inet_openrw(pool *pool, conn_t *c, p_in_addr_t *addr, int fd,
                     int rfd,int wfd, int resolve)
 {
-  conn_t *res;
+  conn_t *res = NULL;
   int close_fd = TRUE;
 
   res = inet_copy_connection(pool,c);
 
   res->listen_fd = -1;
 
-  inet_get_conn_info(res,fd);
+  /* Note: there are some cases where the given file descriptor will
+   * intentionally be bad (e.g. in get_ident() lookups).  In this case,
+   * errno will have a value of EBADF; this is an "acceptable" error.  Any
+   * other errno value constitutes an unacceptable error.
+   */
+  if (inet_get_conn_info(res, fd) < 0 && errno != EBADF)
+    return NULL;
 
-  if(addr) {
-    if(!res->remote_ipaddr)
+  if (addr) {
+    if (!res->remote_ipaddr)
       res->remote_ipaddr = (p_in_addr_t*)palloc(res->pool,sizeof(p_in_addr_t));
     *res->remote_ipaddr = *addr;
   }
 
-  if(resolve && res->remote_ipaddr)
+  if (resolve && res->remote_ipaddr)
     res->remote_name = inet_getname(res->pool,res->remote_ipaddr);
 
-  if(!res->remote_name)
+  if (!res->remote_name)
     res->remote_name = pstrdup(res->pool,inet_ntoa(*res->remote_ipaddr));
 
-  if(fd == -1 && c->listen_fd != -1)
+  if (fd == -1 && c->listen_fd != -1)
     fd = c->listen_fd;
 
-  if(rfd != -1) {
-    if(fd != rfd) dup2(fd,rfd);
-    else close_fd = FALSE;
+  if (rfd != -1) {
+    if (fd != rfd)
+      dup2(fd, rfd);
+    else
+      close_fd = FALSE;
+
   } else
     rfd = dup(fd);
 
-  if(wfd != -1) {
-    if(fd != wfd) {
-      if(wfd == STDOUT_FILENO)
+  if (wfd != -1) {
+    if (fd != wfd) {
+      if (wfd == STDOUT_FILENO)
         fflush(stdout);
-      dup2(fd,wfd);
+      dup2(fd, wfd);
+
     } else
       close_fd = FALSE;
+
   } else
     wfd = dup(fd);
 
   /* Now discard the original socket */
-  if(rfd != -1 && wfd != -1 && close_fd)
+  if (rfd != -1 && wfd != -1 && close_fd)
     close(fd);
 
   res->rfd = rfd;
@@ -1198,7 +1216,7 @@ conn_t *inet_openrw(pool *pool, conn_t *c, p_in_addr_t *addr, int fd,
   res->inf = io_open(res->pool,res->rfd,IO_READ);
   res->outf = io_open(res->pool,res->wfd,IO_WRITE);
 
-  /* Set options on the sockets */
+  /* Set options on the sockets. */
   inet_setoptions(res->pool,res,0,0);
   inet_setblock(res->pool,res);
 
@@ -1221,7 +1239,7 @@ void inet_resolve_ip(pool *pool, conn_t *c)
   }
 }
 
-void init_inet()
+void init_inet(void)
 {
   struct protoent *pr;
 
