@@ -25,7 +25,7 @@
 
 /*
  * House initialization and main program loop
- * $Id: main.c,v 1.69 2001/10/18 17:10:47 flood Exp $
+ * $Id: main.c,v 1.81 2002/05/21 20:47:23 castaglia Exp $
  */
 
 /*
@@ -136,7 +136,7 @@ time_t shut = (time_t)0,deny = (time_t)0, disc = (time_t)0;
 char shutmsg[81] = {'\0'};
 
 xaset_t *children = NULL;
-static int child_count = 0,child_flag = 0;
+static int n_children = 0, have_dead_child = FALSE;
 
 response_t *resp_list = NULL,*resp_err_list = NULL;
 static pool *resp_pool = NULL;
@@ -158,57 +158,6 @@ static int abort_core = 0;
 
 char *config_filename = CONFIG_FILE_PATH;
 
-#if 0
-void test_fs()
-{
-  glob_t pglob;
-  char buf[1024];
-  char **cp;
-  int fd;
-  fsdir_t *f;
-  size_t l;
-
-  bzero(&pglob,sizeof(pglob));
-
-  chdir("/");
-  fs_setcwd("/");
-  if(fs_chdir("~flood/test",TRUE) == -1)
-    perror("fs_chdir [tmp]");
-
-  fs_chdir("test.tar/foo",1);
-
-  printf("cwd: %s\n",fs_getcwd());
-  printf("vwd: %s\n",fs_getvwd());
-
-  f = fs_open("TESTFILE",O_RDONLY,&fd);
-  if(!f) {
-    perror("fs_open");
-    exit(1);
-  }
-
-  fs_lseek(f,fd,-1024,SEEK_END);
-  while((l = fs_read(f,fd,buf,sizeof(buf)-1)) > 0) {
-    buf[l] = '\0';
-    printf("%s",buf);
-  }
-
-  fs_close(f,fd);
-  exit(0);
-
-  if(fs_glob("*",GLOB_PERIOD,0,&pglob) == -1)
-    perror("fs_glob");
-  else {
-    cp = pglob.gl_pathv;
-    while(pglob.gl_pathc--) {
-      printf("matched: %s\n",*cp);
-      cp++;
-    }
-  }
-
-  exit(0);
-}
-#endif
-
 int add_binding(server_rec *server, p_in_addr_t *ipaddr, conn_t *listen,
                 char isdefault, char islocalhost)
 {
@@ -470,7 +419,7 @@ static void set_proc_title(char *fmt,...)
 #endif /* HAVE_SETPROCTITLE */
 }
   
-void main_set_idle()
+void main_set_idle(void)
 {
   time_t now;
 
@@ -650,7 +599,7 @@ void set_auth_check(int (*ck)(cmd_rec*))
   main_check_auth = ck;
 }
 
-void end_login_noexit()
+void end_login_noexit(void)
 {
   /* Run all the exit handlers */
   run_exit_handlers();
@@ -694,9 +643,9 @@ void main_exit(void *pv, void *lv, void *ev, void *dummy)
   if(standalone && master) {
     log_pri(LOG_NOTICE, "ProFTPD %s standalone mode SHUTDOWN", VERSION);
     if(!nodaemon) {
-      PRIVS_ROOT;
+      PRIVS_ROOT
       unlink(PidPath);
-      PRIVS_RELINQUISH;
+      PRIVS_RELINQUISH
     }
   }
   
@@ -769,7 +718,8 @@ static int _dispatch(cmd_rec *cmd, int cmd_type, int validate, char *match)
   static char *last_match = NULL;
   int *index_cache;
 
-  send_error = (cmd_type == CMD || cmd_type == PRE_CMD);
+  send_error = (cmd_type == PRE_CMD || cmd_type == CMD ||
+    cmd_type == POST_CMD_ERR);
 
   if(!match) {
     match = cmd->argv[0];
@@ -822,14 +772,15 @@ static int _dispatch(cmd_rec *cmd, int cmd_type, int validate, char *match)
                        session.user, session.proc_prefix, argstr);
       }
 
-      if (send_error)
-        log_debug(DEBUG4, "dispatching %s command '%s' to mod_%s",
-          (cmd_type == PRE_CMD ? "PRE_CMD" :
-           cmd_type == CMD ? "CMD" :
-           cmd_type == POST_CMD ? "POST_CMD" : 
-           cmd_type == LOG_CMD ? "LOG_CMD" :
-           "(unknown)"),
-          argstr, c->m->name);
+      log_debug(DEBUG4, "dispatching %s command '%s' to mod_%s",
+        (cmd_type == PRE_CMD ? "PRE_CMD" :
+         cmd_type == CMD ? "CMD" :
+         cmd_type == POST_CMD ? "POST_CMD" : 
+         cmd_type == POST_CMD_ERR ? "POST_CMD_ERR" : 
+         cmd_type == LOG_CMD ? "LOG_CMD" :
+         cmd_type == LOG_CMD_ERR ? "LOG_CMD_ERR" :
+         "(unknown)"),
+        argstr, c->m->name);
 
       cmd->class |= c->class;
 
@@ -902,8 +853,16 @@ static void dispatch_cmd(cmd_rec *cmd)
     success = _dispatch(cmd,PRE_CMD,FALSE,NULL);
 
   if(success < 0) {
+
+    /* dispatch to POST_CMD_ERR handlers as well
+     * -tj 2001-08-09
+     */
+    _dispatch(cmd, POST_CMD_ERR, FALSE, "*");
+    _dispatch(cmd, POST_CMD_ERR, FALSE, NULL);
+
     _dispatch(cmd,LOG_CMD_ERR,FALSE,"*");
     _dispatch(cmd,LOG_CMD_ERR,FALSE,NULL);
+
     send_response_list(&resp_err_list);
     return;
   }
@@ -921,7 +880,16 @@ static void dispatch_cmd(cmd_rec *cmd)
     _dispatch(cmd,LOG_CMD,FALSE,NULL);
 
     send_response_list(&resp_list);
+
   } else if(success < 0) {
+
+    /* allow for non-logging command handlers to be run if CMD fails
+     * -tj 2001-08-09
+     */
+    success = _dispatch(cmd, POST_CMD_ERR, FALSE, "*");
+    if (!success)
+      success = _dispatch(cmd, POST_CMD_ERR, FALSE, NULL);
+
     _dispatch(cmd,LOG_CMD_ERR,FALSE,"*");
     _dispatch(cmd,LOG_CMD_ERR,FALSE,NULL);
 
@@ -1145,7 +1113,7 @@ void main_rehash(void *d1,void *d2,void *d3,void *d4)
 
     for(rh = rehash_list; rh; rh=rh->next)
       rh->rehash(rh->data);
-    
+   
     init_config();
     init_conf_stacks();
 
@@ -1256,10 +1224,10 @@ static int _dup_low_fd(int fd)
   return fd;
 }
 
-void fork_server(int fd,conn_t *l,int nofork)
-{
-  server_rec *s,*serv = NULL;
-  conn_t *conn;
+void fork_server(int fd, conn_t *l, int nofork) {
+  server_rec *s = NULL, *s_saved = NULL, *serv = NULL;
+  conn_t *conn = NULL;
+  unsigned char *ident_lookups = NULL;
   int i, rev;
   int sempipe[2] = { -1, -1 };
   
@@ -1348,7 +1316,7 @@ void fork_server(int fd,conn_t *l,int nofork)
       cpid->sempipe = sempipe[0];
       cpid->pool = pidrec_pool;
       xaset_insert(children,(xasetmember_t*)cpid);
-      child_count++;
+      n_children++;
 
       close(sempipe[1]);
       /* Unblock the signals now as sig_child() will catch
@@ -1454,7 +1422,9 @@ void fork_server(int fd,conn_t *l,int nofork)
   /* To conserve memory, free all other servers and associated
    * configurations
    */
-  for(s = main_server; s; s=s->next)
+  s = main_server;
+  while (s) {
+    s_saved = s->next;
     if(s != serv) {
       if(s->listen && s->listen != l) {
 	/* If our former listen socket was stdin or stdout (0 or 1),
@@ -1477,6 +1447,8 @@ void fork_server(int fd,conn_t *l,int nofork)
       xaset_remove(servers,(xasetmember_t*)s);
       destroy_pool(s->pool);
     }
+    s = s_saved;
+  }
 
   main_server = serv;
     
@@ -1564,23 +1536,18 @@ void fork_server(int fd,conn_t *l,int nofork)
     exit(0);
   }
 
-  log_debug(DEBUG4,"connected - local  : %s:%d",
-                    inet_ntoa(*session.c->local_ipaddr),
-                    session.c->local_port);
-  log_debug(DEBUG4,"connected - remote : %s:%d",
-  		    inet_ntoa(*session.c->remote_ipaddr),
-  		    session.c->remote_port);
-
   /* Use the ident protocol (RFC1413) to try to get remote ident_user
    */
 
-  if(get_param_int(main_server->conf,"IdentLookups",FALSE) != 0)
-    session.ident_user = get_ident(session.pool,conn);
-  else
-    session.ident_user = "UNKNOWN";
+  if ((ident_lookups = get_param_ptr(main_server->conf, "IdentLookups",
+     FALSE)) == NULL || *ident_lookups == TRUE) {
+    session.ident_lookups = TRUE;
+    session.ident_user = get_ident(session.pool, conn);
 
-  /* Inform all the modules that we are now a child */
-  init_child_modules();
+  } else {
+    session.ident_lookups = FALSE;
+    session.ident_user = "UNKNOWN";
+  }
 
   /* find class */
   if (get_param_int(main_server->conf, "Classes", FALSE) == 1) {
@@ -1592,11 +1559,21 @@ void fork_server(int fd,conn_t *l,int nofork)
       log_debug(DEBUG2, "FTP session requested from unknown class");
   }
 
+  /* Inform all the modules that we are now a child */
+  init_child_modules();
+
+  log_debug(DEBUG4,"connected - local  : %s:%d",
+                    inet_ntoa(*session.c->local_ipaddr),
+                    session.c->local_port);
+  log_debug(DEBUG4,"connected - remote : %s:%d",
+                    inet_ntoa(*session.c->remote_ipaddr),
+                    session.c->remote_port);
+
   /* xfer_set_data_port(conn->local_ipaddr,conn->local_port-1); */
   cmd_loop(serv,conn);
 }
 
-void disc_children()
+void disc_children(void)
 {
   sigset_t sigset;
   pidrec_t *cp;
@@ -1619,7 +1596,7 @@ void disc_children()
   }
 }
 
-void server_loop()
+void server_loop(void)
 {
   fd_set rfd;
   conn_t *listen;
@@ -1683,7 +1660,7 @@ void server_loop()
 
     i = select(max_fd + 1, &rfd, NULL, NULL,&tv);
 
-    if(child_flag) {
+    if (have_dead_child) {
       sigset_t sigset;
       pidrec_t *cp,*cpnext;
 
@@ -1693,7 +1670,7 @@ void server_loop()
       block_alarms();
       sigprocmask(SIG_BLOCK,&sigset,NULL);
 
-      child_flag = 0;
+      have_dead_child = FALSE;
       if(children) {
         for(cp = (pidrec_t*)children->xas_list; cp; cp=cpnext) {
           cpnext = cp->next;
@@ -1761,7 +1738,7 @@ void server_loop()
 
     listen = accept_binding(&rfd, &fd);
     if(listen) {
-      if(ServerMaxInstances && child_count >= ServerMaxInstances) {
+      if(ServerMaxInstances && n_children >= ServerMaxInstances) {
         log_pri(LOG_WARNING,"MaxInstances (%d) reached, new connection denied.",ServerMaxInstances);
         close(fd);
       } else
@@ -1826,12 +1803,12 @@ static RETSIGTYPE sig_child(int sig)
    */
 
   while((cpid = waitpid(-1,NULL,WNOHANG)) > 0) {
-    child_count--; child_flag = 1;
     if(children) {
       for(cp = (pidrec_t*)children->xas_list; cp; cp=cpnext) {
         cpnext = cp->next;
         if(cp->pid == cpid) {
-          child_flag++;
+          n_children--;
+          have_dead_child = TRUE;
           cp->dead = 1;
         }
       }
@@ -1844,7 +1821,7 @@ static RETSIGTYPE sig_child(int sig)
 }
 
 #ifdef DEBUG_CORE
-static char *_prepare_core()
+static char *_prepare_core(void)
 {
   static char dir[256];
   
@@ -1874,7 +1851,7 @@ static RETSIGTYPE sig_abort(int sig)
 }  
 
 #ifdef DEBUG_CORE
-static void _internal_abort()
+static void _internal_abort(void)
 {
   if(abort_core) {
     log_pri(LOG_NOTICE, "core file dumped to %s", _prepare_core());
@@ -1885,21 +1862,21 @@ static void _internal_abort()
 }
 #endif /* DEBUG_CORE */
 
-static RETSIGTYPE sig_terminate(int sig)
-{
-  pidrec_t *pid;
+static RETSIGTYPE sig_terminate(int sig) {
+  pidrec_t *pid = NULL;
 
   /* If we caught SIGSEGV, restore the default signal handler.
    * jss 3/9/2001
    */
-  if(sig == SIGSEGV)
+  if (sig == SIGSEGV)
     signal(SIGSEGV,SIG_DFL);
   
-  if(sig == SIGTERM) {
+  if (sig == SIGTERM) {
+
     /* Don't log if we are a child that has been terminated */
-    if(master) {
-      /* Send a SIGKILL to all our children */
-      if(children) {
+    if (master) {
+      /* Send a SIGTERM to all our children */
+      if (children) {
         PRIVS_ROOT
         for(pid = (pidrec_t*)children->xas_list; pid; pid=pid->next)
           kill(pid->pid,SIGTERM);
@@ -1915,15 +1892,36 @@ static RETSIGTYPE sig_terminate(int sig)
   } else
     log_pri(LOG_ERR,"ProFTPD terminating (signal %d)",sig);
 
-  if(master && mpid == getpid()) {
+  if (master && mpid == getpid()) {
     PRIVS_ROOT
+
+    /* clean up the scoreboard file */
     log_close_run();
     log_rm_run();
-    if(standalone && !nodaemon)
+
+    /* do not need the pidfile any longer */
+    if (standalone && !nodaemon)
       unlink(PidPath);
+
+    /* run any exit handlers registered in the master process here, so that
+     * they may have the benefit of root privs.  More than likely these
+     * exit handlers were registered by modules' module initialization
+     * functions, which also occur under root priv conditions. (If an
+     * exit handler is registered after the fork(), it won't be run here --
+     * that registration occurs in a different process space.
+     */
+    run_exit_handlers();
+
+    /* remove the registered exit handlers now, so that the ensuing
+     * end_login() call (outside the root privs condition) does not call
+     * the exit handlers for the master process again
+     */
+    remove_exit_handlers();
+
     PRIVS_RELINQUISH
-    if(standalone)
-      log_pri(LOG_NOTICE,"ProFTPD %s standalone mode SHUTDOWN",VERSION);
+
+    if (standalone)
+      log_pri(LOG_NOTICE, "ProFTPD " VERSION " standalone mode SHUTDOWN");
   }
 
 #ifdef DEBUG_CORE
@@ -1933,7 +1931,7 @@ static RETSIGTYPE sig_terminate(int sig)
   end_login(1);
 }
 
-static void install_signal_handlers()
+static void install_signal_handlers(void)
 {
   sigset_t sigset;
 
@@ -1963,7 +1961,9 @@ static void install_signal_handlers()
 #ifdef SIGSTKFLT
   sigaddset(&sigset,SIGSTKFLT);
 #endif
+#ifdef SIGIO
   sigaddset(&sigset,SIGIO);
+#endif
 #ifdef SIGBUS
   sigaddset(&sigset,SIGBUS);
 #endif
@@ -1986,13 +1986,17 @@ static void install_signal_handlers()
 #ifdef SIGSTKFLT
   signal(SIGSTKFLT,sig_terminate);
 #endif /* SIGSTKFLT */
+#ifdef SIGIO
   signal(SIGIO,sig_terminate);
+#endif
 #ifdef SIGBUS
   signal(SIGBUS,sig_terminate);
 #endif /* SIGBUS */
 #endif /* DEBUG_NOSIG */
 
+#ifdef SIGIO
   signal(SIGIO,SIG_IGN);
+#endif
   signal(SIGURG,SIG_IGN);
 
   /* In case our parent left signals blocked (as happens under some
@@ -2001,7 +2005,7 @@ static void install_signal_handlers()
   sigprocmask(SIG_UNBLOCK,&sigset,NULL);
 }
 
-void set_rlimits() {
+void set_rlimits(void) {
   config_rec *c = NULL;
   struct rlimit rlim;
 
@@ -2093,12 +2097,31 @@ void set_rlimits() {
   }
 }
 
-void start_daemon()
+void pr_write_pid(void)
+{
+  FILE *pidf;
+  
+  PidPath = (char *) get_param_ptr(main_server->conf, "PidFile", FALSE);
+  if(!PidPath || !*PidPath)
+    PidPath = PID_FILE_PATH;
+  
+  PRIVS_ROOT
+  if((pidf = fopen(PidPath, "w")) == NULL) {
+    perror(PidPath);
+    exit(1);
+  }
+  
+  fprintf(pidf, "%lu\n", (unsigned long) getpid());
+  fclose(pidf);
+  pidf = NULL;
+  PRIVS_RELINQUISH
+}
+  
+void start_daemon(void)
 {
 #ifndef HAVE_SETSID
   int ttyfd;
 #endif
-  FILE *pidf;
 
   /* Fork off and have parent exit.
    */
@@ -2125,21 +2148,6 @@ void start_daemon()
   }
 #endif /* HAVE_SETSID */
   
-  PidPath = (char *) get_param_ptr(main_server->conf, "PidFile", FALSE);
-  if(!PidPath || !*PidPath)
-    PidPath = PID_FILE_PATH;
-  
-  PRIVS_ROOT;
-  if((pidf = fopen(PidPath, "w")) == NULL) {
-    perror(PidPath);
-    exit(1);
-  }
-  
-  fprintf(pidf, "%lu\n", (unsigned long) getpid());
-  fclose(pidf);
-  pidf = NULL;
-  PRIVS_RELINQUISH;
-  
   /* Close the three big boys */
   close(fileno(stdin));
   close(fileno(stdout));
@@ -2193,7 +2201,7 @@ void addl_bindings(server_rec *s)
   }
 }
 
-void inetd_main()
+void inetd_main(void)
 {
   server_rec *s;
   int isdefault;
@@ -2255,7 +2263,7 @@ void inetd_main()
   fork_server(STDIN_FILENO,main_server->listen,TRUE);
 }
 
-void standalone_main()
+void standalone_main(void)
 {
   server_rec *s;
   int isdefault;
@@ -2340,6 +2348,8 @@ void standalone_main()
 
   log_pri(LOG_NOTICE,"ProFTPD %s (built %s) standalone mode STARTUP",
                       VERSION_STATUS,BUILD_STAMP);
+
+  pr_write_pid();
   server_loop();
 }
 
@@ -2408,8 +2418,8 @@ void show_usage(int exit_code)
 
 int main(int argc, char **argv, char **envp)
 {
-  int socketp;
-  int _umask = 0,c;
+  int socketp, optc;
+  mode_t *main_umask = NULL; 
   int check_config_syntax = 0;
   int show_version = 0;
   struct sockaddr peer;
@@ -2458,13 +2468,12 @@ int main(int argc, char **argv, char **envp)
   tzset();
 #endif
 
-  bzero(&session, sizeof(session));
+  memset(&session, 0, sizeof(session));
 
   /* Initialize stuff for set_proc_title.
    */
   init_set_proc_title(argc, argv, envp);
 
-
   /* Seed rand */
   srand(time(NULL));
 
@@ -2496,14 +2505,14 @@ int main(int argc, char **argv, char **envp)
    */
 
   opterr = 0;
-  while((c =
+  while((optc =
 #ifdef HAVE_GETOPT_LONG
 	 getopt_long(argc, argv, cmdopts, opts, NULL)
 #else /* HAVE_GETOPT_LONG */
 	 getopt(argc, argv, cmdopts)
 #endif /* HAVE_GETOPT_LONG */
 	 ) != -1) {
-    switch(c) {
+    switch (optc) {
     case 'n': 
       nodaemon++;
       break;
@@ -2610,19 +2619,27 @@ int main(int argc, char **argv, char **envp)
   endgrent();
 
   /* Security */
-  daemon_uid = (uid_t) get_param_int(main_server->conf,"User",FALSE);
-  if(daemon_uid == -1)
-    daemon_uid = 0;
-  daemon_gid = (gid_t) get_param_int(main_server->conf,"Group",FALSE);
-  if(daemon_gid == -1)
-    daemon_gid = 0;
+  {
+    uid_t *uid = (uid_t *) get_param_ptr(main_server->conf, "UserID", FALSE);
+    gid_t *gid = (gid_t *) get_param_ptr(main_server->conf, "GroupID", FALSE);
+
+    if (uid)
+      daemon_uid = *uid;
+    else
+      daemon_uid = 0;
+ 
+    if (gid)
+      daemon_gid = *gid;
+    else
+      daemon_gid = 0;
+  }
 
   if (daemon_uid) {
     /* allocate space for daemon supplemental groups
      */
     daemon_gids = make_array(permanent_pool, 2, sizeof(gid_t));
 
-    if (get_groups(permanent_pool, (const char *) get_param_ptr(
+    if (auth_getgroups(permanent_pool, (const char *) get_param_ptr(
         main_server->conf, "UserName", FALSE), &daemon_gids, NULL) < 0)
       log_debug(DEBUG2, "unable to retrieve daemon supplemental groups");
 
@@ -2630,18 +2647,19 @@ int main(int argc, char **argv, char **envp)
       log_pri(LOG_ERR, "unable to set daemon groups: %s",
         strerror(errno));
   }
-  
-   if((_umask = get_param_int(main_server->conf,"Umask",FALSE)) == -1)
-    _umask = 0022;
-
-  umask(_umask);
+ 
+   if ((main_umask = (mode_t *) get_param_ptr(main_server->conf, "Umask",
+       FALSE)) == NULL)
+     umask((mode_t) 0022);
+   else
+     umask(*main_umask);
 
   /* Give up root and save our uid/gid for later use (if supported)
    * If we aren't currently root, PRIVS_SETUP will get rid of setuid
    * granted root and prevent further uid switching from being attempted.
    */
 
-  PRIVS_SETUP(daemon_uid,daemon_gid)
+  PRIVS_SETUP(daemon_uid, daemon_gid)
 
   /* Test to make sure that our uid/gid is correct.  Try to do this in
    * a portable fashion *gah!*
